@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: escape.c,v 1.27 2003/06/26 06:50:32 bagder Exp $
+ * $Id: escape.c,v 1.33 2004/05/13 15:16:36 bagder Exp $
  ***************************************************************************/
 
 /* Escape and unescape URL encoding in strings. The functions return a new
@@ -31,20 +31,23 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "memory.h"
 
 /* The last #include file should be: */
-#ifdef CURLDEBUG
 #include "memdebug.h"
-#endif
 
 char *curl_escape(const char *string, int length)
 {
-  int alloc = (length?length:(int)strlen(string))+1;  
-  char *ns = malloc(alloc);
+  size_t alloc = (length?(size_t)length:strlen(string))+1;  
+  char *ns;
   char *testing_ptr = NULL;
   unsigned char in;
-  int newlen = alloc;
-  int index=0;
+  size_t newlen = alloc;
+  int strindex=0;
+
+  ns = malloc(alloc);
+  if(!ns)
+    return NULL;
 
   length = alloc-1;
   while(length--) {
@@ -65,17 +68,17 @@ char *curl_escape(const char *string, int length)
           ns = testing_ptr;
         }
       }
-      sprintf(&ns[index], "%%%02X", in);
+      sprintf(&ns[strindex], "%%%02X", in);
 
-      index+=3;
+      strindex+=3;
     }
     else {
       /* just copy this */
-      ns[index++]=in;
+      ns[strindex++]=in;
     }
     string++;
   }
-  ns[index]=0; /* terminate it */
+  ns[strindex]=0; /* terminate it */
   return ns;
 }
 
@@ -88,12 +91,11 @@ char *curl_unescape(const char *string, int length)
   int alloc = (length?length:(int)strlen(string))+1;
   char *ns = malloc(alloc);
   unsigned char in;
-  int index=0;
-  unsigned int hex;
+  int strindex=0;
+  long hex;
  
-  if( !ns ) {
+  if( !ns )
     return NULL;
-  }  
   
   while(--alloc > 0) {
     in = *string;
@@ -107,15 +109,15 @@ char *curl_unescape(const char *string, int length)
 
       hex = strtol(hexstr, &ptr, 16);
 
-      in = hex;
+      in = (unsigned char)hex; /* this long is never bigger than 255 anyway */
       string+=2;
       alloc-=2;
     }
     
-    ns[index++] = in;
+    ns[strindex++] = in;
     string++;
   }
-  ns[index]=0; /* terminate it */
+  ns[strindex]=0; /* terminate it */
   return ns;
 }
 
@@ -124,5 +126,6 @@ char *curl_unescape(const char *string, int length)
    the library's memory system */
 void curl_free(void *p)
 {
-  free(p);
+  if(p)
+    free(p);
 }
