@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: formdata.c,v 1.49 2003/06/26 11:22:12 bagder Exp $
+ * $Id: formdata.c,v 1.67 2004/05/13 14:13:12 bagder Exp $
  ***************************************************************************/
 
 /*
@@ -97,14 +97,6 @@ Content-Type: text/plain
 Content-Disposition: form-data; name="FILECONTENT"
 ...
 
-  For the old FormParse used by curl_formparse use:
-
-  gcc -DHAVE_CONFIG_H -I../ -g -D_OLD_FORM_DEBUG -o formdata -I../include formdata.c strequal.c
-
-  run the 'formdata' executable and make sure the output is ok!
-
-  try './formdata "name=Daniel" "poo=noo" "foo=bar"' and similarly
-
  */
 
 #include "setup.h"
@@ -115,275 +107,28 @@ Content-Disposition: form-data; name="FILECONTENT"
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
-
 #include <time.h>
-
 #include <curl/curl.h>
 #include "formdata.h"
-
 #include "strequal.h"
+#include "memory.h"
+
+#define _MPRINTF_REPLACE /* use our functions only */
+#include <curl/mprintf.h>
 
 /* The last #include file should be: */
-#ifdef CURLDEBUG
 #include "memdebug.h"
-#endif
 
 /* Length of the random boundary string. */
 #define BOUNDARY_LENGTH 40
 
 /* What kind of Content-Type to use on un-specified files with unrecognized
    extensions. */
-#define HTTPPOST_CONTENTTYPE_DEFAULT "text/plain"
-
-/* This is a silly duplicate of the function in main.c to enable this source
-   to compile stand-alone for better debugging */
-static void GetStr(char **string,
-		   const char *value)
-{
-  if(*string)
-    free(*string);
-  *string = strdup(value);
-}
-
-/***************************************************************************
- *
- * FormParse()
- *	
- * Reads a 'name=value' paramter and builds the appropriate linked list.
- *
- * Specify files to upload with 'name=@filename'. Supports specified
- * given Content-Type of the files. Such as ';type=<content-type>'.
- *
- * You may specify more than one file for a single name (field). Specify
- * multiple files by writing it like:
- *
- * 'name=@filename,filename2,filename3'
- *
- * If you want content-types specified for each too, write them like:
- *
- * 'name=@filename;type=image/gif,filename2,filename3'
- *
- ***************************************************************************/
+#define HTTPPOST_CONTENTTYPE_DEFAULT "application/octet-stream"
 
 #define FORM_FILE_SEPARATOR ','
 #define FORM_TYPE_SEPARATOR ';'
 
-static
-int FormParse(char *input,
-	      struct curl_httppost **httppost,
-	      struct curl_httppost **last_post)
-{
-  /* nextarg MUST be a string in the format 'name=contents' and we'll
-     build a linked list with the info */
-  char name[256];
-  char *contents;
-  char major[128];
-  char minor[128];
-  long flags = 0;
-  char *contp;
-  const char *type = NULL;
-  char *prevtype = NULL;
-  char *sep;
-  char *sep2;
-  struct curl_httppost *post;
-  struct curl_httppost *subpost; /* a sub-node */
-  unsigned int i;
-
-  /* Preallocate contents to the length of input to make sure we don't
-     overwrite anything. */
-  contents = malloc(strlen(input));
-  contents[0] = '\000';
- 
-  if(1 <= sscanf(input, "%255[^=]=%[^\n]", name, contents)) {
-    /* the input was using the correct format */
-    contp = contents;
-
-    if('@' == contp[0]) {
-      /* we use the @-letter to indicate file name(s) */
-      
-      flags = HTTPPOST_FILENAME;
-      contp++;
-
-      post=NULL;
-
-      do {
-	/* since this was a file, it may have a content-type specifier
-	   at the end too */
-
-	sep=strchr(contp, FORM_TYPE_SEPARATOR);
-	sep2=strchr(contp, FORM_FILE_SEPARATOR);
-
-	/* pick the closest */
-	if(sep2 && (sep2 < sep)) {
-	  sep = sep2;
-
-	  /* no type was specified! */
-	}
-	if(sep) {
-
-	  /* if we got here on a comma, don't do much */
-	  if(FORM_FILE_SEPARATOR != *sep)
-	    type = strstr(sep+1, "type=");
-	  else
-	    type=NULL;
-
-	  *sep=0; /* terminate file name at separator */
-
-	  if(type) {
-	    type += strlen("type=");
-	    
-	    if(2 != sscanf(type, "%127[^/]/%127[^,\n]",
-			   major, minor)) {
-              free(contents);
-	      return 2; /* illegal content-type syntax! */
-	    }
-	    /* now point beyond the content-type specifier */
-	    sep = (char *)type + strlen(major)+strlen(minor)+1;
-
-	    /* find the following comma */
-	    sep=strchr(sep, FORM_FILE_SEPARATOR);
-	  }
-	}
-	else {
-	  type=NULL;
-	  sep=strchr(contp, FORM_FILE_SEPARATOR);
-	}
-	if(sep) {
-	  /* the next file name starts here */
-	  *sep =0;
-	  sep++;
-	}
-	if(!type) {
-	  /*
-	   * No type was specified, we scan through a few well-known
-	   * extensions and pick the first we match!
-	   */
-	  struct ContentType {
-	    const char *extension;
-	    const char *type;
-	  };
-          static struct ContentType ctts[]={
-	    {".gif",  "image/gif"},
-	    {".jpg",  "image/jpeg"},
-	    {".jpeg", "image/jpeg"},
-	    {".txt",  "text/plain"},
-	    {".html", "text/plain"}
-	  };
-
-	  if(prevtype)
-	    /* default to the previously set/used! */
-	    type = prevtype;
-	  else
-	    /* It seems RFC1867 defines no Content-Type to default to
-	       text/plain so we don't actually need to set this: */
-	    type = HTTPPOST_CONTENTTYPE_DEFAULT;
-
-	  for(i=0; i<sizeof(ctts)/sizeof(ctts[0]); i++) {
-	    if(strlen(contp) >= strlen(ctts[i].extension)) {
-	      if(strequal(contp +
-			  strlen(contp) - strlen(ctts[i].extension),
-			  ctts[i].extension)) {
-		type = ctts[i].type;
-		break;
-	      }	      
-	    }
-	  }
-	  /* we have a type by now */
-	}
-
-	if(NULL == post) {
-	  /* For the first file name, we allocate and initiate the main list
-	     node */
-
-	  post = (struct curl_httppost *)malloc(sizeof(struct curl_httppost));
-	  if(post) {
-	    memset(post, 0, sizeof(struct curl_httppost));
-	    GetStr(&post->name, name);      /* get the name */
-	    GetStr(&post->contents, contp); /* get the contents */
-            post->contentslength = 0;
-	    post->flags = flags;
-	    if(type) {
-	      GetStr(&post->contenttype, (char *)type); /* get type */
-	      prevtype=post->contenttype; /* point to the allocated string! */
-	    }
-	    /* make the previous point to this */
-	    if(*last_post)
-	      (*last_post)->next = post;
-	    else
-	      (*httppost) = post;
-
-	    (*last_post) = post;	  
-	  }
-
-	}
-	else {
-	  /* we add a file name to the previously allocated node, known as
-             'post' now */
-	  subpost =(struct curl_httppost *)
-            malloc(sizeof(struct curl_httppost));
-	  if(subpost) {
-	     memset(subpost, 0, sizeof(struct curl_httppost));
-	     GetStr(&subpost->name, name);      /* get the name */
-	     GetStr(&subpost->contents, contp); /* get the contents */
-             subpost->contentslength = 0;
-	     subpost->flags = flags;
-	     if(type) {
-	       GetStr(&subpost->contenttype, (char *)type); /* get type */
-	       prevtype=subpost->contenttype; /* point to allocated string! */
-	     }
-	     /* now, point our 'more' to the original 'more' */
-	     subpost->more = post->more;
-
-	     /* then move the original 'more' to point to ourselves */
-	     post->more = subpost;	     
-	  }
-	}
-	contp = sep; /* move the contents pointer to after the separator */
-      } while(sep && *sep); /* loop if there's another file name */
-    }
-    else {
-      post = (struct curl_httppost *)malloc(sizeof(struct curl_httppost));
-      if(post) {
-	memset(post, 0, sizeof(struct curl_httppost));
-	GetStr(&post->name, name);      /* get the name */
-	if( contp[0]=='<' ) {
-	  GetStr(&post->contents, contp+1); /* get the contents */
-          post->contentslength = 0;
-	  post->flags = HTTPPOST_READFILE;
-	}
-	else {
-	  GetStr(&post->contents, contp); /* get the contents */
-          post->contentslength = 0;
-	  post->flags = 0;
-	}
-
-	/* make the previous point to this */
-	if(*last_post)
-	  (*last_post)->next = post;
-	else
-	  (*httppost) = post;
-
-	(*last_post) = post;	  
-      }
-
-    }
-
-  }
-  else {
-    free(contents);
-    return 1;
-  }
-  free(contents);
-  return 0;
-}
-
-int curl_formparse(char *input,
-                   struct curl_httppost **httppost,
-                   struct curl_httppost **last_post)
-{
-  return FormParse(input, httppost, last_post);
-}
-
 /***************************************************************************
  *
  * AddHttpPost()
@@ -395,12 +140,9 @@ int curl_formparse(char *input,
  *
  ***************************************************************************/
 static struct curl_httppost *
-AddHttpPost(char * name, long namelength,
-            char * value, long contentslength,
-
-            /* CMC: Added support for buffer uploads */
-            char * buffer, long bufferlength,
-
+AddHttpPost(char * name, size_t namelength,
+            char * value, size_t contentslength,
+            char * buffer, size_t bufferlength,
             char *contenttype,
             long flags,
             struct curl_slist* contentHeader,
@@ -410,18 +152,14 @@ AddHttpPost(char * name, long namelength,
             struct curl_httppost **last_post)
 {
   struct curl_httppost *post;
-  post = (struct curl_httppost *)malloc(sizeof(struct curl_httppost));
+  post = (struct curl_httppost *)calloc(sizeof(struct curl_httppost), 1);
   if(post) {
-    memset(post, 0, sizeof(struct curl_httppost));
     post->name = name;
-    post->namelength = name?(namelength?namelength:(long)strlen(name)):0;
+    post->namelength = (long)(name?(namelength?namelength:strlen(name)):0);
     post->contents = value;
-    post->contentslength = contentslength;
-
-    /* CMC: Added support for buffer uploads */
+    post->contentslength = (long)contentslength;
     post->buffer = buffer;
-    post->bufferlength = bufferlength;
-
+    post->bufferlength = (long)bufferlength;
     post->contenttype = contenttype;
     post->contentheader = contentHeader;
     post->showfilename = showfilename;
@@ -544,41 +282,45 @@ static const char * ContentTypeForFilename (const char *filename,
 
 /***************************************************************************
  *
- * AllocAndCopy()
+ * memdup()
  *	
- * Copies the data currently available under *buffer using newly allocated
- * buffer (that becomes *buffer). Uses buffer_length if not null, else
- * uses strlen to determine the length of the buffer to be copied
+ * Copies the 'source' data to a newly allocated buffer buffer (that is
+ * returned). Uses buffer_length if not null, else uses strlen to determine
+ * the length of the buffer to be copied
  *
- * Returns 0 on success and 1 if the malloc failed.
+ * Returns the new pointer or NULL on failure.
  *
  ***************************************************************************/
-static int AllocAndCopy (char **buffer, int buffer_length)
+static char *memdup(const char *src, size_t buffer_length)
 {
-  const char *src = *buffer;
-  int length, add = 0;
+  size_t length;
+  bool add = FALSE;
+  char *buffer;
+  
   if (buffer_length)
     length = buffer_length;
   else {
-    length = strlen(*buffer);
-    add = 1;
+    length = strlen(src);
+    add = TRUE;
   }
-  *buffer = (char*)malloc(length+add);
-  if (!*buffer)
-    return 1;
-  memcpy(*buffer, src, length);
+  buffer = (char*)malloc(length+add);
+  if (!buffer)
+    return NULL; /* fail */
+  
+  memcpy(buffer, src, length);
+
   /* if length unknown do null termination */
   if (add)
-    (*buffer)[length] = '\0';
-  return 0;
+    buffer[length] = '\0';
+
+  return buffer;
 }
 
 /***************************************************************************
  *
  * FormAdd()
  *	
- * Stores a 'name=value' formpost parameter and builds the appropriate
- * linked list.
+ * Stores a formpost parameter and builds the appropriate linked list.
  *
  * Has two principal functionalities: using files and byte arrays as
  * post parts. Byte arrays are either copied or just the pointer is stored
@@ -628,7 +370,7 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
                      struct curl_httppost **last_post,
                      va_list params)
 {
-  FormInfo *first_form, *current_form, *form;
+  FormInfo *first_form, *current_form, *form = NULL;
   CURLFORMcode return_value = CURL_FORMADD_OK;
   const char *prevtype = NULL;
   struct curl_httppost *post = NULL;
@@ -644,22 +386,16 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
   /*
    * We need to allocate the first struct to fill in.
    */
-  first_form = (FormInfo *)malloc(sizeof(struct FormInfo));
-  if(first_form) {
-    memset(first_form, 0, sizeof(FormInfo));
-    current_form = first_form;
-  }
-  else
+  first_form = (FormInfo *)calloc(sizeof(struct FormInfo), 1);
+  if(!first_form)
     return CURL_FORMADD_MEMORY;
 
-  /*
-   * Loop through all the options set.
-   */
-  while (1) {
+  current_form = first_form;
 
-    /* break if we have an error to report */
-    if (return_value != CURL_FORMADD_OK)
-      break;
+  /*
+   * Loop through all the options set. Break if we have an error to report.
+   */
+  while (return_value == CURL_FORMADD_OK) {
 
     /* first see if we have more parts of the array param */
     if ( array_state ) {
@@ -754,7 +490,12 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
           array_value:va_arg(params, char *);
         if (filename) {
           current_form->value = strdup(filename);
-          current_form->flags |= HTTPPOST_READFILE;
+          if(!current_form->value)
+            return_value = CURL_FORMADD_MEMORY;
+          else {
+            current_form->flags |= HTTPPOST_READFILE;
+            current_form->value_alloc = TRUE;
+          }
         }
         else
           return_value = CURL_FORMADD_NULL;
@@ -781,16 +522,21 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
             return_value = CURL_FORMADD_OPTION_TWICE;
         }
         else {
-          if (filename)
+          if (filename) {
             current_form->value = strdup(filename);
+            if(!current_form->value)
+              return_value = CURL_FORMADD_MEMORY;
+            else {
+              current_form->flags |= HTTPPOST_FILENAME;
+              current_form->value_alloc = TRUE;
+            }
+          }
           else
             return_value = CURL_FORMADD_NULL;
-          current_form->flags |= HTTPPOST_FILENAME;
         }
         break;
       }
 
-    /* CMC: Added support for buffer uploads */
     case CURLFORM_BUFFER:
       {
         char *filename = array_state?array_value:
@@ -810,8 +556,11 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
             return_value = CURL_FORMADD_OPTION_TWICE;
         }
         else {
-          if (filename)
+          if (filename) {
             current_form->value = strdup(filename);
+            if(!current_form->value)
+              return_value = CURL_FORMADD_MEMORY;
+          }
           else
             return_value = CURL_FORMADD_NULL;
           current_form->flags |= HTTPPOST_BUFFER;
@@ -819,7 +568,6 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
         break;
       }
       
-    /* CMC: Added support for buffer uploads */
     case CURLFORM_BUFFERPTR:
         current_form->flags |= HTTPPOST_PTRBUFFER;
       if (current_form->buffer)
@@ -834,7 +582,6 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
       }
       break;
 
-    /* CMC: Added support for buffer uploads */
     case CURLFORM_BUFFERLENGTH:
       if (current_form->bufferlength)
         return_value = CURL_FORMADD_OPTION_TWICE;
@@ -862,8 +609,13 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
             return_value = CURL_FORMADD_OPTION_TWICE;
         }
         else {
-	  if (contenttype)
+	  if (contenttype) {
 	    current_form->contenttype = strdup(contenttype);
+            if(!current_form->contenttype)
+              return_value = CURL_FORMADD_MEMORY;
+            else
+              current_form->contenttype_alloc = TRUE;
+          }
 	  else
 	    return_value = CURL_FORMADD_NULL;
 	}
@@ -890,8 +642,13 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
           va_arg(params, char *);
         if( current_form->showfilename )
           return_value = CURL_FORMADD_OPTION_TWICE;
-        else
+        else {
           current_form->showfilename = strdup(filename);
+          if(!current_form->showfilename)
+            return_value = CURL_FORMADD_MEMORY;
+          else
+            current_form->showfilename_alloc = TRUE;
+        }
         break;
       }
     default:
@@ -913,7 +670,6 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
            ( (form->flags & HTTPPOST_FILENAME) &&
              (form->flags & HTTPPOST_PTRCONTENTS) ) ||
 
-           /* CMC: Added support for buffer uploads */
            ( (!form->buffer) &&
              (form->flags & HTTPPOST_BUFFER) &&
              (form->flags & HTTPPOST_PTRBUFFER) ) ||
@@ -931,41 +687,46 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
           /* our contenttype is missing */
           form->contenttype
             = strdup(ContentTypeForFilename(form->value, prevtype));
+          if(!form->contenttype) {
+            return_value = CURL_FORMADD_MEMORY;
+            break;
+          }
+          form->contenttype_alloc = TRUE;
         }
         if ( !(form->flags & HTTPPOST_PTRNAME) &&
              (form == first_form) ) {
           /* copy name (without strdup; possibly contains null characters) */
-          if (AllocAndCopy(&form->name, form->namelength)) {
+          form->name = memdup(form->name, form->namelength);
+          if (!form->name) {
             return_value = CURL_FORMADD_MEMORY;
             break;
           }
+          form->name_alloc = TRUE;
         }
         if ( !(form->flags & HTTPPOST_FILENAME) &&
              !(form->flags & HTTPPOST_READFILE) && 
              !(form->flags & HTTPPOST_PTRCONTENTS) &&
-
-             /* CMC: Added support for buffer uploads */
              !(form->flags & HTTPPOST_PTRBUFFER) ) {
-
           /* copy value (without strdup; possibly contains null characters) */
-          if (AllocAndCopy(&form->value, form->contentslength)) {
+          form->value = memdup(form->value, form->contentslength);
+          if (!form->value) {
             return_value = CURL_FORMADD_MEMORY;
             break;
           }
+          form->value_alloc = TRUE;
         }
         post = AddHttpPost(form->name, form->namelength,
                            form->value, form->contentslength,
-
-                           /* CMC: Added support for buffer uploads */
                            form->buffer, form->bufferlength,
-
                            form->contenttype, form->flags,
                            form->contentheader, form->showfilename,
                            post, httppost,
                            last_post);
         
-        if(!post)
+        if(!post) {
           return_value = CURL_FORMADD_MEMORY;
+          break;
+        }
 
         if (form->contenttype)
           prevtype = form->contenttype;
@@ -973,6 +734,22 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
     }
   }
 
+  if(return_value) {
+    /* we return on error, free possibly allocated fields */
+    if(!form)
+      form = current_form;
+    if(form) {
+      if(form->name_alloc)
+        free(form->name);
+      if(form->value_alloc)
+        free(form->value);
+      if(form->contenttype_alloc)
+        free(form->contenttype);
+      if(form->showfilename_alloc)
+        free(form->showfilename);
+    }
+  }
+
   /* always delete the allocated memory before returning */
   form = first_form;
   while (form != NULL) {
@@ -986,9 +763,13 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
   return return_value;
 }
 
+/*
+ * curl_formadd() is a public API to add a section to the multipart formpost.
+ */
+
 CURLFORMcode curl_formadd(struct curl_httppost **httppost,
-                 struct curl_httppost **last_post,
-                 ...)
+                          struct curl_httppost **last_post,
+                          ...)
 {
   va_list arg;
   CURLFORMcode result;
@@ -998,12 +779,20 @@ CURLFORMcode curl_formadd(struct curl_httppost **httppost,
   return result;
 }
 
-static int AddFormData(struct FormData **formp,
-                       const void *line,
-                       long length)
+/*
+ * AddFormData() adds a chunk of data to the FormData linked list.
+ *
+ * size is incremented by the chunk length, unless it is NULL
+ */
+static CURLcode AddFormData(struct FormData **formp,
+                            const void *line,
+                            size_t length,
+                            size_t *size)
 {
   struct FormData *newform = (struct FormData *)
     malloc(sizeof(struct FormData));
+  if (!newform)
+    return CURLE_OUT_OF_MEMORY;
   newform->next = NULL;
 
   /* we make it easier for plain strings: */
@@ -1011,6 +800,10 @@ static int AddFormData(struct FormData **formp,
     length = strlen((char *)line);
 
   newform->line = (char *)malloc(length+1);
+  if (!newform->line) {
+    free(newform);
+    return CURLE_OUT_OF_MEMORY;
+  }
   memcpy(newform->line, line, length);
   newform->length = length;
   newform->line[length]=0; /* zero terminate for easier debugging */
@@ -1022,12 +815,18 @@ static int AddFormData(struct FormData **formp,
   else
     *formp = newform;
 
-  return length;
+  if (size)
+    *size += length;
+  return CURLE_OK;
 }
 
+/*
+ * AddFormDataf() adds printf()-style formatted data to the formdata chain.
+ */
 
-static int AddFormDataf(struct FormData **formp,
-                        const char *fmt, ...)
+static CURLcode AddFormDataf(struct FormData **formp,
+                             size_t *size,
+                             const char *fmt, ...)
 {
   char s[4096];
   va_list ap;
@@ -1035,16 +834,19 @@ static int AddFormDataf(struct FormData **formp,
   vsprintf(s, fmt, ap);
   va_end(ap);
 
-  return AddFormData(formp, s, 0);
+  return AddFormData(formp, s, 0, size);
 }
 
-
+/*
+ * Curl_FormBoundary() creates a suitable boundary string and returns an
+ * allocated one.
+ */
 char *Curl_FormBoundary(void)
 {
   char *retstring;
   static int randomizer=0; /* this is just so that two boundaries within
 			      the same form won't be identical */
-  int i;
+  size_t i;
 
   static char table16[]="abcdef0123456789";
 
@@ -1067,11 +869,17 @@ char *Curl_FormBoundary(void)
   return retstring;
 }
 
-/* Used from http.c, this cleans a built FormData linked list */ 
+/*
+ * Curl_formclean() is used from http.c, this cleans a built FormData linked
+ * list
+ */ 
 void Curl_formclean(struct FormData *form)
 {
   struct FormData *next;
 
+  if(!form)
+    return;
+
   do {
     next=form->next;  /* the following form line */
     free(form->line); /* free the line */
@@ -1080,7 +888,10 @@ void Curl_formclean(struct FormData *form)
   } while((form=next)); /* continue */
 }
 
-/* external function to free up a whole form post chain */
+/*
+ * curl_formfree() is an external function to free up a whole form post
+ * chain
+ */
 void curl_formfree(struct curl_httppost *form)
 {
   struct curl_httppost *next;
@@ -1109,16 +920,23 @@ void curl_formfree(struct curl_httppost *form)
   } while((form=next)); /* continue */
 }
 
+/*
+ * Curl_getFormData() converts a linked list of "meta data" into a complete
+ * (possibly huge) multipart formdata. The input list is in 'post', while the
+ * output resulting linked lists gets stored in '*finalform'. *sizep will get
+ * the total size of the whole POST.
+ */
+
 CURLcode Curl_getFormData(struct FormData **finalform,
                           struct curl_httppost *post,
-                          int *sizep)
+                          curl_off_t *sizep)
 {
   struct FormData *form = NULL;
   struct FormData *firstform;
   struct curl_httppost *file;
   CURLcode result = CURLE_OK;
 
-  int size =0;
+  size_t size =0;
   char *boundary;
   char *fileboundary=NULL;
   struct curl_slist* curList;
@@ -1129,30 +947,48 @@ CURLcode Curl_getFormData(struct FormData **finalform,
     return result; /* no input => no output! */
 
   boundary = Curl_FormBoundary();
+  if(!boundary)
+    return CURLE_OUT_OF_MEMORY;
   
   /* Make the first line of the output */
-  AddFormDataf(&form,
-               "Content-Type: multipart/form-data;"
-               " boundary=%s\r\n",
-               boundary);
-  /* we DO NOT count that line since that'll be part of the header! */
+  result = AddFormDataf(&form, NULL,
+                        "Content-Type: multipart/form-data;"
+                        " boundary=%s\r\n",
+                        boundary);
+  if (result) {
+    free(boundary);
+    return result;
+  }
+  /* we DO NOT include that line in the total size of the POST, since it'll be
+     part of the header! */
 
   firstform = form;
   
   do {
 
-    if(size)
-      size += AddFormDataf(&form, "\r\n");
+    if(size) {
+      result = AddFormDataf(&form, &size, "\r\n");
+      if (result)
+        break;
+    }
 
     /* boundary */
-    size += AddFormDataf(&form, "--%s\r\n", boundary);
+    result = AddFormDataf(&form, &size, "--%s\r\n", boundary);
+    if (result)
+      break;
 
-    size += AddFormData(&form,
-                        "Content-Disposition: form-data; name=\"", 0);
+    result = AddFormData(&form,
+                         "Content-Disposition: form-data; name=\"", 0, &size);
+    if (result)
+      break;
 
-    size += AddFormData(&form, post->name, post->namelength);
+    result = AddFormData(&form, post->name, post->namelength, &size);
+    if (result)
+      break;
 
-    size += AddFormData(&form, "\"", 0);
+    result = AddFormData(&form, "\"", 0, &size);
+    if (result)
+      break;
 
     if(post->more) {
       /* If used, this is a link to more file names, we must then do
@@ -1160,10 +996,12 @@ CURLcode Curl_getFormData(struct FormData **finalform,
 
       fileboundary = Curl_FormBoundary();
 
-      size += AddFormDataf(&form,
-                           "\r\nContent-Type: multipart/mixed,"
-                           " boundary=%s\r\n",
-                           fileboundary);
+      result = AddFormDataf(&form, &size,
+                            "\r\nContent-Type: multipart/mixed,"
+                            " boundary=%s\r\n",
+                            fileboundary);
+      if (result)
+        break;
     }
 
     file = post;
@@ -1176,37 +1014,48 @@ CURLcode Curl_getFormData(struct FormData **finalform,
 
       if(post->more) {
         /* if multiple-file */
-        size += AddFormDataf(&form,
-                             "\r\n--%s\r\nContent-Disposition: "
-                             "attachment; filename=\"%s\"",
-                             fileboundary,
-                             (file->showfilename?file->showfilename:
-                              file->contents));
+        result = AddFormDataf(&form, &size,
+                              "\r\n--%s\r\nContent-Disposition: "
+                              "attachment; filename=\"%s\"",
+                              fileboundary,
+                              (file->showfilename?file->showfilename:
+                               file->contents));
+        if (result)
+          break;
       }
       else if((post->flags & HTTPPOST_FILENAME) ||
-
-              /* CMC: Added support for buffer uploads */
               (post->flags & HTTPPOST_BUFFER)) {
 
-        size += AddFormDataf(&form,
-                             "; filename=\"%s\"",
-                             (post->showfilename?post->showfilename:
-                              post->contents));
+        result = AddFormDataf(&form, &size,
+                              "; filename=\"%s\"",
+                              (post->showfilename?post->showfilename:
+                               post->contents));
+        if (result)
+          break;
       }
       
       if(file->contenttype) {
         /* we have a specified type */
-        size += AddFormDataf(&form,
-                             "\r\nContent-Type: %s",
-                             file->contenttype);
+        result = AddFormDataf(&form, &size,
+                              "\r\nContent-Type: %s",
+                              file->contenttype);
+        if (result)
+          break;
       }
 
       curList = file->contentheader;
       while( curList ) {
         /* Process the additional headers specified for this form */
-        size += AddFormDataf( &form, "\r\n%s", curList->data );
+        result = AddFormDataf( &form, &size, "\r\n%s", curList->data );
+        if (result)
+          break;
         curList = curList->next;
       }
+      if (result) {
+        Curl_formclean(firstform);
+        free(boundary);
+        return result;
+      }
 
 #if 0
       /* The header Content-Transfer-Encoding: seems to confuse some receivers
@@ -1222,65 +1071,97 @@ CURLcode Curl_getFormData(struct FormData **finalform,
       }
 #endif
 
-      size += AddFormData(&form, "\r\n\r\n", 0);
+      result = AddFormData(&form, "\r\n\r\n", 0, &size);
+      if (result)
+        break;
 
       if((post->flags & HTTPPOST_FILENAME) ||
          (post->flags & HTTPPOST_READFILE)) {
         /* we should include the contents from the specified file */
         FILE *fileread;
         char buffer[1024];
-        int nread;
+        size_t nread;
+
+        fileread = strequal("-", file->contents)?
+          stdin:fopen(file->contents, "rb"); /* binary read for win32  */
+        /*
+         * VMS: This only allows for stream files on VMS.  Stream files are
+         * OK, as are FIXED & VAR files WITHOUT implied CC For implied CC,
+         * every record needs to have a \n appended & 1 added to SIZE
+         */
 
-        fileread = strequal("-", file->contents)?stdin:
-          /* binary read for win32 crap */
-          /*VMS??*/ fopen(file->contents, "rb");  /* ONLY ALLOWS FOR STREAM FILES ON VMS */
-        /*VMS?? Stream files are OK, as are FIXED & VAR files WITHOUT implied CC */
-        /*VMS?? For implied CC, every record needs to have a \n appended & 1 added to SIZE */
         if(fileread) {
-          while((nread = fread(buffer, 1, 1024, fileread)))
-            size += AddFormData(&form, buffer, nread);
-
+          while((nread = fread(buffer, 1, 1024, fileread))) {
+            result = AddFormData(&form, buffer, nread, &size);
+            if (result)
+              break;
+          }
           if(fileread != stdin)
             fclose(fileread);
+          if (result) {
+            Curl_formclean(firstform);
+            free(boundary);
+            return result;
+          }
+
         }
         else {
-#if 0
-          /* File wasn't found, add a nothing field! */
-          size += AddFormData(&form, "", 0);
-#endif
           Curl_formclean(firstform);
           free(boundary);
           *finalform = NULL;
           return CURLE_READ_ERROR;
         }
 
-        /* CMC: Added support for buffer uploads */
-      } else if (post->flags & HTTPPOST_BUFFER) {
-          /* include contents of buffer */
-          size += AddFormData(&form, post->buffer, post->bufferlength);
+      } 
+      else if (post->flags & HTTPPOST_BUFFER) {
+        /* include contents of buffer */
+        result = AddFormData(&form, post->buffer, post->bufferlength,
+                             &size);
+          if (result)
+            break;
       }
 
       else {
         /* include the contents we got */
-        size += AddFormData(&form, post->contents, post->contentslength);
+        result = AddFormData(&form, post->contents, post->contentslength,
+                             &size);
+        if (result)
+          break;
       }
     } while((file = file->more)); /* for each specified file for this field */
+    if (result) {
+      Curl_formclean(firstform);
+      free(boundary);
+      return result;
+    }
 
     if(post->more) {
       /* this was a multiple-file inclusion, make a termination file
          boundary: */
-      size += AddFormDataf(&form,
+      result = AddFormDataf(&form, &size,
                            "\r\n--%s--",
                            fileboundary);     
       free(fileboundary);
+      if (result)
+        break;
     }
 
   } while((post=post->next)); /* for each field */
+  if (result) {
+    Curl_formclean(firstform);
+    free(boundary);
+    return result;
+  }
 
   /* end-boundary for everything */
-  size += AddFormDataf(&form,
+  result = AddFormDataf(&form, &size,
                        "\r\n--%s--\r\n",
                        boundary);
+  if (result) {
+    Curl_formclean(firstform);
+    free(boundary);
+    return result;
+  }
 
   *sizep = size;
 
@@ -1291,6 +1172,10 @@ CURLcode Curl_getFormData(struct FormData **finalform,
   return result;
 }
 
+/*
+ * Curl_FormInit() inits the struct 'form' points to with the 'formdata'
+ * and resets the 'sent' counter.
+ */
 int Curl_FormInit(struct Form *form, struct FormData *formdata )
 {
   if(!formdata)
@@ -1302,15 +1187,18 @@ int Curl_FormInit(struct Form *form, struct FormData *formdata )
   return 0;
 }
 
-/* fread() emulation */
-int Curl_FormReader(char *buffer,
-                    size_t size,
-                    size_t nitems,
-                    FILE *mydata)
+/*
+ * Curl_FormReader() is the fread() emulation function that will be used to
+ * deliver the formdata to the transfer loop and then sent away to the peer.
+ */
+size_t Curl_FormReader(char *buffer,
+                       size_t size,
+                       size_t nitems,
+                       FILE *mydata)
 {
   struct Form *form;
-  int wantedsize;
-  int gotsize = 0;
+  size_t wantedsize;
+  size_t gotsize = 0;
 
   form=(struct Form *)mydata;
 
@@ -1348,48 +1236,25 @@ int Curl_FormReader(char *buffer,
   return gotsize;
 }
 
-/* possible (old) fread() emulation that copies at most one line */
-int Curl_FormReadOneLine(char *buffer,
-                         size_t size,
-                         size_t nitems,
-                         FILE *mydata)
+/*
+ * Curl_formpostheader() returns the first line of the formpost, the
+ * request-header part (which is not part of the request-body like the rest of
+ * the post).
+ */
+char *Curl_formpostheader(void *formp, size_t *len)
 {
-  struct Form *form;
-  int wantedsize;
-  int gotsize;
-
-  form=(struct Form *)mydata;
-
-  wantedsize = size * nitems;
+  char *header;
+  struct Form *form=(struct Form *)formp;
 
   if(!form->data)
-    return -1; /* nothing, error, empty */
+    return 0; /* nothing, ERROR! */
 
-  do {
-  
-    if( (form->data->length - form->sent ) > wantedsize ) {
+  header = form->data->line;
+  *len = form->data->length;
 
-      memcpy(buffer, form->data->line + form->sent, wantedsize);
+  form->data = form->data->next; /* advance */
 
-      form->sent += wantedsize;
-
-      return wantedsize;
-    }
-
-    memcpy(buffer,
-           form->data->line + form->sent,
-           gotsize = (form->data->length - form->sent) );
-
-    form->sent = 0;
-
-    form->data = form->data->next; /* advance */
-
-  } while(!gotsize && form->data);
-  /* If we got an empty line and we have more data, we proceed to the next
-     line immediately to avoid returning zero before we've reached the end.
-     This is the bug reported November 22 1999 on curl 6.3. (Daniel) */
-
-  return gotsize;
+  return header;
 }
 
 
@@ -1440,7 +1305,7 @@ int main()
   int value6length = strlen(value5);
   int errors = 0;
   int size;
-  int nread;
+  size_t nread;
   char buffer[4096];
   struct curl_httppost *httppost=NULL;
   struct curl_httppost *last_post=NULL;
