@@ -8,7 +8,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -21,36 +21,37 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: formdata.h,v 1.15 2003/01/16 21:08:13 bagder Exp $
+ * $Id: formdata.h,v 1.22 2004/05/12 09:02:23 bagder Exp $
  ***************************************************************************/
 /* plain and simple linked list with lines to send */
 struct FormData {
   struct FormData *next;
   char *line;
-  long length;
+  size_t length;
 };
 
 struct Form {
   struct FormData *data; /* current form line to send */
-  int sent; /* number of bytes of the current line that has already
-	       been sent in a previous invoke */
+  unsigned int sent; /* number of bytes of the current line that has already
+                        been sent in a previous invoke */
 };
 
 /* used by FormAdd for temporary storage */
 typedef struct FormInfo {
   char *name;
-  long namelength;
+  bool name_alloc;
+  size_t namelength;
   char *value;
-  long contentslength;
+  bool value_alloc;
+  size_t contentslength;
   char *contenttype;
+  bool contenttype_alloc;
   long flags;
-
-		/* CMC: Added support for buffer uploads */
   char *buffer;      /* pointer to existing buffer used for file upload */
-	long bufferlength;   
-
+  size_t bufferlength;
   char *showfilename; /* The file name to show. If not set, the actual
                          file name will be used */
+  bool showfilename_alloc;
   struct curl_slist* contentheader;
   struct FormInfo *more;
 } FormInfo;
@@ -59,20 +60,21 @@ int Curl_FormInit(struct Form *form, struct FormData *formdata );
 
 CURLcode
 Curl_getFormData(struct FormData **,
-                 struct HttpPost *post,
-                 int *size);
+                 struct curl_httppost *post,
+                 curl_off_t *size);
 
 /* fread() emulation */
-int Curl_FormReader(char *buffer,
-                    size_t size,
-                    size_t nitems,
-                    FILE *mydata);
+size_t Curl_FormReader(char *buffer,
+                       size_t size,
+                       size_t nitems,
+                       FILE *mydata);
 
-/* possible (old) fread() emulation that copies at most one line */
-int Curl_FormReadOneLine(char *buffer,
-                         size_t size,
-                         size_t nitems,
-                         FILE *mydata);
+/*
+ * Curl_formpostheader() returns the first line of the formpost, the
+ * request-header part (which is not part of the request-body like the rest of
+ * the post).
+ */
+char *Curl_formpostheader(void *formp, size_t *len);
 
 char *Curl_FormBoundary(void);
 
