@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: getenv.c,v 1.18 2002/09/03 11:53:00 bagder Exp $
+ * $Id: getenv.c,v 1.24 2004/05/11 11:30:23 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -35,9 +35,10 @@
 #include <unixlib.h>
 #endif
 
-#ifdef MALLOCDEBUG
+#include <curl/curl.h>
+#include "memory.h"
+
 #include "memdebug.h"
-#endif
 
 static
 char *GetEnv(const char *variable)
@@ -67,11 +68,3 @@ char *curl_getenv(const char *v)
 {
   return GetEnv(v);
 }
-
-/*
- * local variables:
- * eval: (load-file "../curl-mode.el")
- * end:
- * vim600: fdm=marker
- * vim: et sw=2 ts=2 sts=2 tw=78
- */
