@@ -7,7 +7,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -20,13 +20,13 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: if2ip.h,v 1.12 2003/02/28 13:11:10 bagder Exp $
+ * $Id: if2ip.h,v 1.16 2004/05/05 13:42:23 bagder Exp $
  ***************************************************************************/
 #include "setup.h"
 
-#if ! defined(WIN32) && ! defined(__BEOS__) && !defined(__CYGWIN32__) && \
-    ! defined(__riscos__)
-extern char *Curl_if2ip(char *interface, char *buf, int buf_size);
+#if !defined(WIN32) && !defined(__BEOS__) && !defined(__CYGWIN32__) && \
+    !defined(__riscos__) && !defined(__INTERIX)
+extern char *Curl_if2ip(const char *interface, char *buf, int buf_size);
 #else
 #define Curl_if2ip(a,b,c) NULL
 #endif
