@@ -6,7 +6,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -19,20 +19,16 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: memdebug.c,v 1.30 2003/08/14 14:19:36 bagder Exp $
+ * $Id: memdebug.c,v 1.44 2004/05/11 11:30:23 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
 
 #include <curl/curl.h>
 
-#if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
-#include <winsock.h>
-#else /* some kind of unix */
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
-#endif
 
 #define _MPRINTF_REPLACE
 #include <curl/mprintf.h>
@@ -45,10 +41,12 @@
 #include <unistd.h>
 #endif
 
-/* DONT include memdebug.h here! */
+#define MEMDEBUG_NODEFINES /* don't redefine the standard functions */
+#include "memory.h"
+#include "memdebug.h"
 
 struct memdebug {
-  int size;
+  size_t size;
   double mem[1];
   /* I'm hoping this is the thing with the strictest alignment
    * requirements.  That also means we waste some space :-( */
@@ -94,6 +92,9 @@ static bool countcheck(const char *func, int line, const char *source)
       if(logfile && source)
         fprintf(logfile, "LIMIT %s:%d %s reached memlimit\n",
                 source, line, func);
+      if(source)
+        fprintf(stderr, "LIMIT %s:%d %s reached memlimit\n",
+                source, line, func);
       return TRUE; /* RETURN ERROR! */
     }
     else
@@ -120,7 +121,7 @@ void *curl_domalloc(size_t wantedsize, int line, const char *source)
   /* alloc at least 64 bytes */
   size = sizeof(struct memdebug)+wantedsize;
 
-  mem=(struct memdebug *)(malloc)(size);
+  mem=(struct memdebug *)(Curl_cmalloc)(size);
   if(mem) {
     /* fill memory with junk */
     memset(mem->mem, 0xA5, wantedsize);
@@ -128,52 +129,78 @@ void *curl_domalloc(size_t wantedsize, int line, const char *source)
   }
 
   if(logfile && source)
-    fprintf(logfile, "MEM %s:%d malloc(%d) = %p\n",
-            source, line, wantedsize, mem->mem);
-  return mem->mem;
+    fprintf(logfile, "MEM %s:%d malloc(%zd) = %p\n",
+            source, line, wantedsize, mem ? mem->mem : 0);
+  return (mem ? mem->mem : NULL);
+}
+
+void *curl_docalloc(size_t wanted_elements, size_t wanted_size,
+                    int line, const char *source)
+{
+  struct memdebug *mem;
+  size_t size, user_size;
+
+  if(countcheck("calloc", line, source))
+    return NULL;
+
+  /* alloc at least 64 bytes */
+  user_size = wanted_size * wanted_elements;
+  size = sizeof(struct memdebug) + user_size;
+
+  mem = (struct memdebug *)(Curl_cmalloc)(size);
+  if(mem) {
+    /* fill memory with zeroes */
+    memset(mem->mem, 0, user_size);
+    mem->size = user_size;
+  }
+
+  if(logfile && source)
+    fprintf(logfile, "MEM %s:%d calloc(%u,%u) = %p\n",
+            source, line, wanted_elements, wanted_size, mem ? mem->mem : 0);
+  return (mem ? mem->mem : NULL);
 }
 
 char *curl_dostrdup(const char *str, int line, const char *source)
 {
   char *mem;
   size_t len;
-  
-  if(NULL ==str) {
-    fprintf(stderr, "ILLEGAL strdup() on NULL at %s:%d\n",
-            source, line);
-    exit(2);
-  }
 
+  curlassert(str != NULL);
+  
   if(countcheck("strdup", line, source))
     return NULL;
 
   len=strlen(str)+1;
 
   mem=curl_domalloc(len, 0, NULL); /* NULL prevents logging */
+  if (mem)
   memcpy(mem, str, len);
 
   if(logfile)
-    fprintf(logfile, "MEM %s:%d strdup(%p) (%d) = %p\n",
+    fprintf(logfile, "MEM %s:%d strdup(%p) (%zd) = %p\n",
             source, line, str, len, mem);
 
   return mem;
 }
 
+/* We provide a realloc() that accepts a NULL as pointer, which then
+   performs a malloc(). In order to work with ares. */
 void *curl_dorealloc(void *ptr, size_t wantedsize,
                      int line, const char *source)
 {
-  struct memdebug *mem;
+  struct memdebug *mem=NULL;
 
   size_t size = sizeof(struct memdebug)+wantedsize;
 
   if(countcheck("realloc", line, source))
     return NULL;
 
-  mem = (struct memdebug *)((char *)ptr - offsetof(struct memdebug, mem));
+  if(ptr)
+    mem = (struct memdebug *)((char *)ptr - offsetof(struct memdebug, mem));
 
-  mem=(struct memdebug *)(realloc)(mem, size);
+  mem=(struct memdebug *)(Curl_crealloc)(mem, size);
   if(logfile)
-    fprintf(logfile, "MEM %s:%d realloc(%p, %d) = %p\n",
+    fprintf(logfile, "MEM %s:%d realloc(0x%x, %zd) = %p\n",
             source, line, ptr, wantedsize, mem?mem->mem:NULL);
 
   if(mem) {
@@ -188,35 +215,35 @@ void curl_dofree(void *ptr, int line, const char *source)
 {
   struct memdebug *mem;
 
-  if(NULL == ptr) {
-    fprintf(stderr, "ILLEGAL free() on NULL at %s:%d\n",
-            source, line);
-    exit(2);
-  }
+  curlassert(ptr != NULL);
+
   mem = (struct memdebug *)((char *)ptr - offsetof(struct memdebug, mem));
 
   /* destroy  */
   memset(mem->mem, 0x13, mem->size);
   
   /* free for real */
-  (free)(mem);
+  (Curl_cfree)(mem);
 
   if(logfile)
     fprintf(logfile, "MEM %s:%d free(%p)\n", source, line, ptr);
 }
 
-int curl_socket(int domain, int type, int protocol, int line, char *source)
+int curl_socket(int domain, int type, int protocol, int line,
+                const char *source)
 {
   int sockfd=(socket)(domain, type, protocol);
-  if(logfile)
+  if(logfile && (sockfd!=-1))
     fprintf(logfile, "FD %s:%d socket() = %d\n",
             source, line, sockfd);
   return sockfd;
 }
 
-int curl_accept(int s, struct sockaddr *addr, socklen_t *addrlen,
+int curl_accept(int s, void *saddr, void *saddrlen,
                 int line, const char *source)
 {
+  struct sockaddr *addr = (struct sockaddr *)saddr;
+  socklen_t *addrlen = (socklen_t *)saddrlen;
   int sockfd=(accept)(s, addr, addrlen);
   if(logfile)
     fprintf(logfile, "FD %s:%d accept() = %d\n",
@@ -225,7 +252,7 @@ int curl_accept(int s, struct sockaddr *addr, socklen_t *addrlen,
 }
 
 /* this is our own defined way to close sockets on *ALL* platforms */
-int curl_sclose(int sockfd, int line, char *source)
+int curl_sclose(int sockfd, int line, const char *source)
 {
   int res=sclose(sockfd);
   if(logfile)
@@ -239,8 +266,8 @@ FILE *curl_fopen(const char *file, const char *mode,
 {
   FILE *res=(fopen)(file, mode);
   if(logfile)
-    fprintf(logfile, "FILE %s:%d fopen(\"%s\") = %p\n",
-            source, line, file, res);
+    fprintf(logfile, "FILE %s:%d fopen(\"%s\",\"%s\") = %p\n",
+            source, line, file, mode, res);
   return res;
 }
 
@@ -248,11 +275,7 @@ int curl_fclose(FILE *file, int line, const char *source)
 {
   int res;
 
-  if(NULL == file) {
-    fprintf(stderr, "ILLEGAL flose() on NULL at %s:%d\n",
-            source, line);
-    exit(2);
-  }
+  curlassert(file != NULL);
 
   res=(fclose)(file);
   if(logfile)
@@ -263,5 +286,8 @@ int curl_fclose(FILE *file, int line, const char *source)
 #else
 #ifdef VMS
 int VOID_VAR_MEMDEBUG;	
-#endif
+#else
+/* we provide a fake do-nothing function here to avoid compiler warnings */
+void curl_memdebug(void) {}
+#endif /* VMS */
 #endif /* CURLDEBUG */
