@@ -1,6 +1,6 @@
 /****************************************************************************
  *
- * $Id: mprintf.c,v 1.25 2003/08/06 22:32:47 bagder Exp $
+ * $Id: mprintf.c,v 1.43 2004/05/12 12:05:13 bagder Exp $
  *
  *************************************************************************
  *
@@ -38,20 +38,27 @@
 #include <ctype.h>
 #include <string.h>
 
-#ifndef SIZEOF_LONG_LONG
-/* prevents warnings on picky compilers */
-#define SIZEOF_LONG_LONG 0
-#endif
+#include <curl/mprintf.h>
+
 #ifndef SIZEOF_LONG_DOUBLE
 #define SIZEOF_LONG_DOUBLE 0
 #endif
 
-
-/* The last #include file should be: */
-#ifdef CURLDEBUG
-#include "memdebug.h"
+#ifndef SIZEOF_SIZE_T
+/* default to 4 bytes for size_t unless defined in the config.h */
+#define SIZEOF_SIZE_T 4
 #endif
 
+#ifdef DPRINTF_DEBUG
+#define HAVE_LONGLONG
+#define LONG_LONG long long
+#define ENABLE_64BIT
+#endif
+
+#include "memory.h"
+/* The last #include file should be: */
+#include "memdebug.h"
+
 #define BUFFSIZE 256 /* buffer for long-to-str and float-to-str calcs */
 #define MAX_PARAMETERS 128 /* lame static limit */
 
@@ -75,7 +82,13 @@ static const char lower_digits[] = "0123456789abcdefghijklmnopqrstuvwxyz";
 /* Upper-case digits.  */
 static const char upper_digits[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
-#define	OUTCHAR(x) done+=(stream(x, (FILE *)data)==-1?0:1)
+#define	OUTCHAR(x) \
+  do{ \
+    if(stream((unsigned char)(x), (FILE *)data) != -1) \
+      done++; \
+    else \
+     return done; /* return immediately on failure */ \
+  } while(0)
 
 /* Data type to read from the arglist */
 typedef enum  {
@@ -119,17 +132,17 @@ enum {
 typedef struct {
   FormatType type;
   int flags;
-  int width;     /* width OR width parameter number */
-  int precision; /* precision OR precision parameter number */
+  long width;     /* width OR width parameter number */
+  long precision; /* precision OR precision parameter number */
   union {
     char *str;
     void *ptr;
     long num;
-#if SIZEOF_LONG_LONG /* if this is non-zero */
-    long long lnum;
+#ifdef ENABLE_64BIT
+    LONG_LONG lnum;
 #endif
     double dnum;
-#if SIZEOF_LONG_DOUBLE
+#if 0 /*SIZEOF_LONG_DOUBLE */
     long double ldnum;
 #endif
   } data;
@@ -145,11 +158,13 @@ struct asprintf {
   char *buffer; /* allocated buffer */
   size_t len;   /* length of string */
   size_t alloc; /* length of alloc */
+  bool fail;    /* TRUE if an alloc has failed and thus the output is not
+                   the complete data */
 };
 
 int curl_msprintf(char *buffer, const char *format, ...);
 
-static int dprintf_DollarString(char *input, char **end)
+static long dprintf_DollarString(char *input, char **end)
 {
   int number=0;
   while(isdigit((int)*input)) {
@@ -170,8 +185,8 @@ static BOOL dprintf_IsQualifierNoDollar(char c)
   case '-': case '+': case ' ': case '#': case '.':
   case '0': case '1': case '2': case '3': case '4':
   case '5': case '6': case '7': case '8': case '9':
-  case 'h': case 'l': case 'L': case 'Z': case 'q':
-  case '*':
+  case 'h': case 'l': case 'L': case 'z': case 'q':
+  case '*': case 'O':
     return TRUE;
   default:
     return FALSE;
@@ -278,16 +293,17 @@ int dprintf_Pass1Report(va_stack_t *vto, int max)
  *
  ******************************************************************/
 
-static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list arglist)
+static long dprintf_Pass1(char *format, va_stack_t *vto, char **endpos,
+                          va_list arglist)
 {
   char *fmt = format;
   int param_num = 0;
-  int this_param;
-  int width;
-  int precision;
+  long this_param;
+  long width;
+  long precision;
   int flags;
-  int max_param=0;
-  int i;
+  long max_param=0;
+  long i;
 
   while (*fmt) {
     if (*fmt++ == '%') {
@@ -374,12 +390,22 @@ static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list a
 	case 'q':
 	  flags |= FLAGS_LONGLONG;
 	  break;
-	case 'Z':
-	  if (sizeof(size_t) > sizeof(unsigned long int))
-	    flags |= FLAGS_LONGLONG;
-	  if (sizeof(size_t) > sizeof(unsigned int))
-	    flags |= FLAGS_LONG;
-	  break;
+	case 'z':
+          /* the code below generates a warning if -Wunreachable-code is
+             used */
+#if SIZEOF_SIZE_T>4
+          flags |= FLAGS_LONGLONG;
+#else
+          flags |= FLAGS_LONG;
+#endif
+          break;
+        case 'O':
+#if SIZEOF_CURL_OFF_T > 4
+          flags |= FLAGS_LONGLONG;
+#else
+          flags |= FLAGS_LONG;
+#endif
+          break;
 	case '0':
 	  if (!(flags & FLAGS_LEFT))
 	    flags |= FLAGS_PAD_NIL;
@@ -449,13 +475,21 @@ static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list a
       case 'f':
 	vto[i].type = FORMAT_DOUBLE;
 	break;
-      case 'e': case 'E':
+      case 'e':
 	vto[i].type = FORMAT_DOUBLE;
-	flags |= FLAGS_FLOATE| (('E' == *fmt)?FLAGS_UPPER:0);
+	flags |= FLAGS_FLOATE;
 	break;
-      case 'g': case 'G':
+      case 'E':
 	vto[i].type = FORMAT_DOUBLE;
-	flags |= FLAGS_FLOATG| (('G' == *fmt)?FLAGS_UPPER:0);
+	flags |= FLAGS_FLOATE|FLAGS_UPPER;
+	break;
+      case 'g':
+	vto[i].type = FORMAT_DOUBLE;
+	flags |= FLAGS_FLOATG;
+	break;	
+      case 'G':
+	vto[i].type = FORMAT_DOUBLE;
+	flags |= FLAGS_FLOATG|FLAGS_UPPER;
 	break;	
       default:
 	vto[i].type = FORMAT_UNKNOWN;
@@ -517,9 +551,9 @@ static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list a
 	break;
 	
       case FORMAT_INT:
-#if SIZEOF_LONG_LONG
+#ifdef ENABLE_64BIT
 	if(vto[i].flags & FLAGS_LONGLONG)
-	  vto[i].data.lnum = va_arg(arglist, long long);
+	  vto[i].data.lnum = va_arg(arglist, LONG_LONG);
 	else
 #endif
 	  if(vto[i].flags & FLAGS_LONG)
@@ -529,7 +563,7 @@ static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list a
 	break;
 	
       case FORMAT_DOUBLE:
-#if SIZEOF_LONG_DOUBLE
+#if 0 /*SIZEOF_LONG_DOUBLE */
 	if(vto[i].flags & FLAGS_LONG)
 	  vto[i].data.ldnum = va_arg(arglist, long double);
 	else
@@ -554,12 +588,12 @@ static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list a
 }
 
 static int dprintf_formatf(
-             void *data, /* untouched by format(), just sent to the
-                            stream() function in the first argument */
-	     int (*stream)(int, FILE *), /* function pointer called for each
-					    output character */
-	     const char *format,    /* %-formatted string */
-	     va_list ap_save) /* list of parameters */
+  void *data, /* untouched by format(), just sent to the stream() function in
+                 the second argument */
+  /* function pointer called for each output character */
+  int (*stream)(int, FILE *),
+  const char *format,    /* %-formatted string */
+  va_list ap_save) /* list of parameters */
 {
   /* Base-36 digits for numbers.  */
   const char *digits = lower_digits;
@@ -568,7 +602,7 @@ static int dprintf_formatf(
   char *f;
 
   /* Number of characters written.  */
-  register size_t done = 0;
+  int done = 0;
 
   long param; /* current parameter to read */
   long param_num=0; /* parameter counter */
@@ -593,7 +627,8 @@ static int dprintf_formatf(
     char alt;
     
     /* Width of a field.  */
-    register long width;
+    long width;
+
     /* Precision of a field.  */
     long prec;
     
@@ -604,8 +639,8 @@ static int dprintf_formatf(
     long base;
 
     /* Integral values to be written.  */
-#if SIZEOF_LONG_LONG
-    unsigned long long num;
+#ifdef ENABLE_64BIT
+    unsigned LONG_LONG num;
 #else
     unsigned long num;
 #endif
@@ -660,7 +695,7 @@ static int dprintf_formatf(
     else
       prec = -1;
 
-    alt = p->flags & FLAGS_ALT;
+    alt = (p->flags & FLAGS_ALT)?TRUE:FALSE;
     
     switch (p->type) {
     case FORMAT_INT:
@@ -697,9 +732,9 @@ static int dprintf_formatf(
       /* Decimal integer.  */
       base = 10;
 
-#if SIZEOF_LONG_LONG
+#ifdef ENABLE_64BIT
       if(p->flags & FLAGS_LONGLONG) {
-	 /* long long */
+        /* long long */
 	is_neg = p->data.lnum < 0;
 	num = is_neg ? (- p->data.lnum) : p->data.lnum;
       }
@@ -721,7 +756,7 @@ static int dprintf_formatf(
       /* Number of base BASE.  */
       {
 	char *workend = &work[sizeof(work) - 1];
-	register char *w;
+	char *w;
 	
 	/* Supply a default precision if none was given.  */
 	if (prec == -1)
@@ -850,7 +885,7 @@ static int dprintf_formatf(
 	else {
 	  /* Write "(nil)" for a nil pointer.  */
 	  static char strnil[] = "(nil)";
-	  register char *point;
+	  char *point;
 	  
 	  width -= sizeof(strnil) - 1;
 	  if (p->flags & FLAGS_LEFT)
@@ -895,11 +930,11 @@ static int dprintf_formatf(
 
 	if(width >= 0) {
 	  /* RECURSIVE USAGE */
-	  fptr += curl_msprintf(fptr, "%d", width);
+	  fptr += curl_msprintf(fptr, "%ld", width);
 	}
 	if(prec >= 0) {
 	  /* RECURSIVE USAGE */
-	  fptr += curl_msprintf(fptr, ".%d", prec);
+	  fptr += curl_msprintf(fptr, ".%ld", prec);
 	}
 	if (p->flags & FLAGS_LONG)
 	  strcat(fptr, "l");
@@ -913,7 +948,7 @@ static int dprintf_formatf(
 
 	/* NOTE NOTE NOTE!! Not all sprintf() implementations returns number
 	   of output characters */
-#if SIZEOF_LONG_DOUBLE
+#if 0 /*SIZEOF_LONG_DOUBLE*/
 	if (p->flags & FLAGS_LONG)
 	  /* This is for support of the 'long double' type */
 	  (sprintf)(work, formatbuf, p->data.ldnum);
@@ -928,17 +963,17 @@ static int dprintf_formatf(
 
     case FORMAT_INTPTR:
       /* Answer the count of characters written.  */
-#if SIZEOF_LONG_LONG
+#ifdef ENABLE_64BIT
       if (p->flags & FLAGS_LONGLONG)
-	*(long long int *) p->data.ptr = done;
+	*(LONG_LONG *) p->data.ptr = (LONG_LONG)done;
       else
 #endif
 	if (p->flags & FLAGS_LONG)
-	  *(long int *) p->data.ptr = done;
+	  *(long *) p->data.ptr = (long)done;
       else if (!(p->flags & FLAGS_SHORT))
-	*(int *) p->data.ptr = done;
+	*(int *) p->data.ptr = (int)done;
       else
-	*(short int *) p->data.ptr = done;
+	*(short *) p->data.ptr = (short)done;
       break;
 
     default:
@@ -954,61 +989,62 @@ static int dprintf_formatf(
 static int addbyter(int output, FILE *data)
 {
   struct nsprintf *infop=(struct nsprintf *)data;
+  unsigned char outc = (unsigned char)output;
  
   if(infop->length < infop->max) {
     /* only do this if we haven't reached max length yet */
-    infop->buffer[0] = (char)output; /* store */
+    infop->buffer[0] = outc; /* store */
     infop->buffer++; /* increase pointer */
     infop->length++; /* we are now one byte larger */
-    return output; /* fputc() returns like this on success */
+    return outc;     /* fputc() returns like this on success */
   }
   return -1;
 }
 
+int curl_mvsnprintf(char *buffer, size_t maxlength, const char *format,
+                    va_list ap_save)
+{
+  int retcode;
+  struct nsprintf info;
+
+  info.buffer = buffer;
+  info.length = 0;
+  info.max = maxlength;
+
+  retcode = dprintf_formatf(&info, addbyter, format, ap_save);
+  if(info.max) {
+    /* we terminate this with a zero byte */
+    if(info.max == info.length)
+      /* we're at maximum, scrap the last letter */
+      info.buffer[-1] = 0;
+    else
+      info.buffer[0] = 0;
+  }
+  return retcode;
+}
+
 int curl_msnprintf(char *buffer, size_t maxlength, const char *format, ...)
 {
+  int retcode;
   va_list ap_save; /* argument pointer */
-  int retcode;
-  struct nsprintf info;
-
-  info.buffer = buffer;
-  info.length = 0;
-  info.max = maxlength;
-
   va_start(ap_save, format);
-  retcode = dprintf_formatf(&info, addbyter, format, ap_save);
+  retcode = curl_mvsnprintf(buffer, maxlength, format, ap_save);
   va_end(ap_save);
-  info.buffer[0] = 0; /* we terminate this with a zero byte */
-
-  /* we could even return things like */
-  
   return retcode;
 }
 
-int curl_mvsnprintf(char *buffer, size_t maxlength, const char *format, va_list ap_save)
-{
-  int retcode;
-  struct nsprintf info;
-
-  info.buffer = buffer;
-  info.length = 0;
-  info.max = maxlength;
-
-  retcode = dprintf_formatf(&info, addbyter, format, ap_save);
-  info.buffer[0] = 0; /* we terminate this with a zero byte */
-  return retcode;
-}
-
-
 /* fputc() look-alike */
 static int alloc_addbyter(int output, FILE *data)
 {
   struct asprintf *infop=(struct asprintf *)data;
+  unsigned char outc = (unsigned char)output;
  
   if(!infop->buffer) {
     infop->buffer=(char *)malloc(32);
-    if(!infop->buffer)
+    if(!infop->buffer) {
+      infop->fail = TRUE;
       return -1; /* fail */
+    }
     infop->alloc = 32;
     infop->len =0;
   }
@@ -1018,17 +1054,18 @@ static int alloc_addbyter(int output, FILE *data)
     newptr = (char *)realloc(infop->buffer, infop->alloc*2);
 
     if(!newptr) {
+      infop->fail = TRUE;
       return -1;
     }
     infop->buffer = newptr;
     infop->alloc *= 2;
   }
 
-  infop->buffer[ infop->len ] = output;
+  infop->buffer[ infop->len ] = outc;
 
   infop->len++;
 
-  return output; /* fputc() returns like this on success */
+  return outc; /* fputc() returns like this on success */
 }
 
 char *curl_maprintf(const char *format, ...)
@@ -1040,11 +1077,12 @@ char *curl_maprintf(const char *format, ...)
   info.buffer = NULL;
   info.len = 0;
   info.alloc = 0;
+  info.fail = FALSE;
 
   va_start(ap_save, format);
   retcode = dprintf_formatf(&info, alloc_addbyter, format, ap_save);
   va_end(ap_save);
-  if(-1 == retcode) {
+  if((-1 == retcode) || info.fail) {
     if(info.alloc)
       free(info.buffer);
     return NULL;
@@ -1065,9 +1103,10 @@ char *curl_mvaprintf(const char *format, va_list ap_save)
   info.buffer = NULL;
   info.len = 0;
   info.alloc = 0;
+  info.fail = FALSE;
 
   retcode = dprintf_formatf(&info, alloc_addbyter, format, ap_save);
-  if(-1 == retcode) {
+  if((-1 == retcode) || info.fail) {
     if(info.alloc)
       free(info.buffer);
     return NULL;
@@ -1084,9 +1123,10 @@ char *curl_mvaprintf(const char *format, va_list ap_save)
 static int storebuffer(int output, FILE *data)
 {
   char **buffer = (char **)data;
-  **buffer = (char)output;
+  unsigned char outc = (unsigned char)output;
+  **buffer = outc;
   (*buffer)++;
-  return output; /* act like fputc() ! */
+  return outc; /* act like fputc() ! */
 }
 
 int curl_msprintf(char *buffer, const char *format, ...)
@@ -1147,14 +1187,16 @@ int main()
 {
   char buffer[129];
   char *ptr;
-#if SIZEOF_LONG_LONG>0
-  long long hullo;
-  dprintf("%3$12s %1$s %2$qd %4$d\n", "daniel", hullo, "stenberg", 65);
+#ifdef ENABLE_64BIT
+  long long one=99;
+  long long two=100;
+  long long test = 0x1000000000LL;
+  curl_mprintf("%lld %lld %lld\n", one, two, test);
 #endif
 
-  mprintf("%3d %5d\n", 10, 1998);
+  curl_mprintf("%3d %5d\n", 10, 1998);
   
-  ptr=maprintf("test this then baby %s%s%s%s%s%s %d %d %d loser baby get a hit in yer face now!", "", "pretty long string pretty long string pretty long string pretty long string pretty long string", "/", "/", "/", "pretty long string", 1998, 1999, 2001);
+  ptr=curl_maprintf("test this then baby %s%s%s%s%s%s %d %d %d loser baby get a hit in yer face now!", "", "pretty long string pretty long string pretty long string pretty long string pretty long string", "/", "/", "/", "pretty long string", 1998, 1999, 2001);
 
   puts(ptr);
 
@@ -1163,15 +1205,15 @@ int main()
   free(ptr);
 
 #if 1
-  mprintf(buffer, "%s %s %d", "daniel", "stenberg", 19988);
+  curl_mprintf(buffer, "%s %s %d", "daniel", "stenberg", 19988);
   puts(buffer);
 
-  mfprintf(stderr, "%s %#08x\n", "dummy", 65);
+  curl_mfprintf(stderr, "%s %#08x\n", "dummy", 65);
 
   printf("%s %#08x\n", "dummy", 65);
   {
     double tryout = 3.14156592;
-    mprintf(buffer, "%.2g %G %f %e %E", tryout, tryout, tryout, tryout, tryout);
+    curl_mprintf(buffer, "%.2g %G %f %e %E", tryout, tryout, tryout, tryout, tryout);
     puts(buffer);
     printf("%.2g %G %f %e %E\n", tryout, tryout, tryout, tryout, tryout);
   }
