@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sendf.c,v 1.67 2003/08/11 09:56:06 bagder Exp $
+ * $Id: sendf.c,v 1.84 2004/05/11 11:30:23 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -39,9 +39,6 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
-#include <winsock.h>
-#endif
 
 #include <curl/curl.h>
 #include "urldata.h"
@@ -51,14 +48,13 @@
 #define _MPRINTF_REPLACE /* use the internal *printf() functions */
 #include <curl/mprintf.h>
 
-#ifdef KRB4
+#ifdef HAVE_KRB4
 #include "security.h"
 #endif
 #include <string.h>
+#include "memory.h"
 /* The last #include file should be: */
-#ifdef CURLDEBUG
 #include "memdebug.h"
-#endif
 
 /* returns last node in linked list */
 static struct curl_slist *slist_get_last(struct curl_slist *list)
@@ -77,11 +73,13 @@ static struct curl_slist *slist_get_last(struct curl_slist *list)
   return item;
 }
 
-/* append a struct to the linked list. It always retunrs the address of the
- * first record, so that you can sure this function as an initialization
- * function as well as an append function. If you find this bothersome,
- * then simply create a separate _init function and call it appropriately from
- * within the proram. */
+/*
+ * curl_slist_append() appends a string to the linked list. It always retunrs
+ * the address of the first record, so that you can sure this function as an
+ * initialization function as well as an append function. If you find this
+ * bothersome, then simply create a separate _init function and call it
+ * appropriately from within the proram.
+ */
 struct curl_slist *curl_slist_append(struct curl_slist *list,
                                      const char *data)
 {
@@ -90,12 +88,18 @@ struct curl_slist *curl_slist_append(struct curl_slist *list,
 
   new_item = (struct curl_slist *) malloc(sizeof(struct curl_slist));
   if (new_item) {
-    new_item->next = NULL;
-    new_item->data = strdup(data);
+    char *dup = strdup(data);
+    if(dup) {
+      new_item->next = NULL;
+      new_item->data = dup;
+    }
+    else {
+      free(new_item);
+      return NULL;
+    }
   }
-  if (new_item == NULL || new_item->data == NULL) {
+  else
     return NULL;
-  }
 
   if (list) {
     last = slist_get_last(list);
@@ -155,9 +159,9 @@ void Curl_failf(struct SessionHandle *data, const char *fmt, ...)
     data->state.errorbuf = TRUE; /* wrote error string */
 
     if(data->set.verbose) {
-      int len = strlen(data->set.errorbuffer);
+      size_t len = strlen(data->set.errorbuffer);
       bool doneit=FALSE;
-      if(len < CURL_ERROR_SIZE) {
+      if(len < CURL_ERROR_SIZE - 1) {
         doneit = TRUE;
         data->set.errorbuffer[len] = '\n';
         data->set.errorbuffer[++len] = '\0';
@@ -172,12 +176,12 @@ void Curl_failf(struct SessionHandle *data, const char *fmt, ...)
 }
 
 /* Curl_sendf() sends formated data to the server */
-CURLcode Curl_sendf(int sockfd, struct connectdata *conn,
+CURLcode Curl_sendf(curl_socket_t sockfd, struct connectdata *conn,
                     const char *fmt, ...)
 {
   struct SessionHandle *data = conn->data;
   ssize_t bytes_written;
-  ssize_t write_len;
+  size_t write_len;
   CURLcode res;
   char *s;
   char *sptr;
@@ -192,7 +196,7 @@ CURLcode Curl_sendf(int sockfd, struct connectdata *conn,
   write_len = strlen(s);
   sptr = s;
 
-  do {
+  while (1) {
     /* Write the buffer to the socket */
     res = Curl_write(conn, sockfd, sptr, write_len, &bytes_written);
 
@@ -202,7 +206,7 @@ CURLcode Curl_sendf(int sockfd, struct connectdata *conn,
     if(data->set.verbose)
       Curl_debug(data, CURLINFO_DATA_OUT, sptr, bytes_written);
 
-    if(bytes_written != write_len) {
+    if((size_t)bytes_written != write_len) {
       /* if not all was written at once, we must advance the pointer, decrease
          the size left and try again! */
       write_len -= bytes_written;
@@ -210,8 +214,7 @@ CURLcode Curl_sendf(int sockfd, struct connectdata *conn,
     }
     else
       break;
-
-  } while(1);
+  }
 
   free(s); /* free the output string */
 
@@ -221,25 +224,32 @@ CURLcode Curl_sendf(int sockfd, struct connectdata *conn,
 /*
  * Curl_write() is an internal write function that sends plain (binary) data
  * to the server. Works with plain sockets, SSL or kerberos.
- *
  */
-CURLcode Curl_write(struct connectdata *conn, int sockfd,
-                    void *mem, size_t len,
+CURLcode Curl_write(struct connectdata *conn,
+                    curl_socket_t sockfd,
+                    void *mem,
+                    size_t len,
                     ssize_t *written)
 {
   ssize_t bytes_written;
   CURLcode retcode;
-  (void)conn;
 
 #ifdef USE_SSLEAY
+  /* Set 'num' to 0 or 1, depending on which socket that has been sent here.
+     If it is the second socket, we set num to 1. Otherwise to 0. This lets
+     us use the correct ssl handle. */
+  int num = (sockfd == conn->sock[SECONDARYSOCKET]);
   /* SSL_write() is said to return 'int' while write() and send() returns
      'size_t' */
-  if (conn->ssl.use) {
+  if (conn->ssl[num].use) {
     int err;
-    int rc = SSL_write(conn->ssl.handle, mem, len);
+    char error_buffer[120]; /* OpenSSL documents that this must be at least
+                               120 bytes long. */
+    unsigned long sslerror;
+    int rc = SSL_write(conn->ssl[num].handle, mem, len);
 
     if(rc < 0) {
-      err = SSL_get_error(conn->ssl.handle, rc);
+      err = SSL_get_error(conn->ssl[num].handle, rc);
     
       switch(err) {
       case SSL_ERROR_WANT_READ:
@@ -254,18 +264,12 @@ CURLcode Curl_write(struct connectdata *conn, int sockfd,
               Curl_ourerrno());
         return CURLE_SEND_ERROR;
       case SSL_ERROR_SSL:
-      {
-        /*  A failure in the SSL library occurred, usually a
-            protocol error.  The OpenSSL error queue contains more
-            information on the error. */
-        char error_buffer[120]; /* OpenSSL documents that this must be at least
-                                   120 bytes long. */
-        int sslerror = ERR_get_error();
+        /*  A failure in the SSL library occurred, usually a protocol error.
+            The OpenSSL error queue contains more information on the error. */
+        sslerror = ERR_get_error();
         failf(conn->data, "SSL_write() error: %s\n",
               ERR_error_string(sslerror, error_buffer));
         return CURLE_SEND_ERROR;
-      }
-        break;
       }
       /* a true error */
       failf(conn->data, "SSL_write() return error %d\n", err);
@@ -274,15 +278,17 @@ CURLcode Curl_write(struct connectdata *conn, int sockfd,
     bytes_written = rc;
   }
   else {
+#else
+  (void)conn;
 #endif
-#ifdef KRB4
+#ifdef HAVE_KRB4
     if(conn->sec_complete) {
       bytes_written = Curl_sec_write(conn, sockfd, mem, len);
     }
     else
-#endif /* KRB4 */
+#endif /* HAVE_KRB4 */
     {
-      bytes_written = swrite(sockfd, mem, len);
+      bytes_written = (ssize_t)swrite(sockfd, mem, len);
     }
     if(-1 == bytes_written) {
       int err = Curl_ourerrno();
@@ -360,23 +366,27 @@ CURLcode Curl_client_write(struct SessionHandle *data,
  * If the read would block (EWOULDBLOCK) we return -1. Otherwise we return
  * a regular CURLcode value.
  */
-int Curl_read(struct connectdata *conn,
-              int sockfd,
-              char *buf,
-              size_t buffersize,
-              ssize_t *n)
+int Curl_read(struct connectdata *conn, /* connection data */
+              curl_socket_t sockfd,     /* read from this socket */
+              char *buf,                /* store read data here */
+              size_t buffersize,        /* max amount to read */
+              ssize_t *n)               /* amount bytes read */
 {
   ssize_t nread;
-  (void)conn;
+#ifdef USE_SSLEAY
+  /* Set 'num' to 0 or 1, depending on which socket that has been sent here.
+     If it is the second socket, we set num to 1. Otherwise to 0. This lets
+     us use the correct ssl handle. */
+  int num = (sockfd == conn->sock[SECONDARYSOCKET]);
+
   *n=0; /* reset amount to zero */
 
-#ifdef USE_SSLEAY
-  if (conn->ssl.use) {
-    nread = SSL_read(conn->ssl.handle, buf, buffersize);
+  if (conn->ssl[num].use) {
+    nread = SSL_read(conn->ssl[num].handle, buf, buffersize);
 
     if(nread < 0) {
       /* failed SSL_read */
-      int err = SSL_get_error(conn->ssl.handle, nread);
+      int err = SSL_get_error(conn->ssl[num].handle, nread);
 
       switch(err) {
       case SSL_ERROR_NONE: /* this is not an error */
@@ -386,31 +396,31 @@ int Curl_read(struct connectdata *conn,
       case SSL_ERROR_WANT_WRITE:
         /* there's data pending, re-invoke SSL_read() */
         return -1; /* basicly EWOULDBLOCK */
-      case SSL_ERROR_SYSCALL:
-        /* openssl/ssl.h says "look at error stack/return value/errno" */
-      {
-        char error_buffer[120]; /* OpenSSL documents that this must be at least
-                                   120 bytes long. */
-        int sslerror = ERR_get_error();
-        failf(conn->data, "SSL read: %s, errno %d",
-              ERR_error_string(sslerror, error_buffer),
-              Curl_ourerrno() );
-      }
-      return CURLE_RECV_ERROR;
       default:
-        failf(conn->data, "SSL read error: %d", err);
+        /* openssl/ssl.h says "look at error stack/return value/errno" */
+        {
+          char error_buffer[120]; /* OpenSSL documents that this must be at
+                                     least 120 bytes long. */
+          unsigned long sslerror = ERR_get_error();
+          failf(conn->data, "SSL read: %s, errno %d",
+                ERR_error_string(sslerror, error_buffer),
+                Curl_ourerrno() );
+        }
         return CURLE_RECV_ERROR;
       }
     }
   }
   else {
+#else
+    (void)conn;
 #endif
-#ifdef KRB4
+    *n=0; /* reset amount to zero */
+#ifdef HAVE_KRB4
     if(conn->sec_complete)
       nread = Curl_sec_read(conn, sockfd, buf, buffersize);
     else
 #endif
-      nread = sread (sockfd, buf, buffersize);
+      nread = sread(sockfd, buf, buffersize);
 
     if(-1 == nread) {
       int err = Curl_ourerrno();
