@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: telnet.c,v 1.43 2003/07/25 08:30:58 bagder Exp $
+ * $Id: telnet.c,v 1.61 2004/05/12 12:06:39 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -35,8 +35,7 @@
 
 #include <errno.h>
 
-#if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
-#include <winsock2.h>
+#if defined(WIN32)
 #include <time.h>
 #include <io.h>
 #else
@@ -73,6 +72,7 @@
 #include <curl/curl.h>
 #include "transfer.h"
 #include "sendf.h"
+#include "telnet.h"
 
 #define _MPRINTF_REPLACE /* use our functions only */
 #include <curl/mprintf.h>
@@ -81,11 +81,10 @@
 #define  TELCMDS
 
 #include "arpa_telnet.h"
+#include "memory.h"
 
 /* The last #include file should be: */
-#ifdef CURLDEBUG
 #include "memdebug.h"
-#endif
 
 #define SUBBUFSIZE 512
 
@@ -101,6 +100,11 @@
 #define  CURL_SB_EOF(x) (x->subpointer >= x->subend)
 #define  CURL_SB_LEN(x) (x->subend - x->subpointer)
 
+#ifdef WIN32
+typedef FARPROC WSOCK2_FUNC;
+static CURLcode check_wsock2 ( struct SessionHandle *data );
+#endif
+
 static
 void telrcv(struct connectdata *,
 	    unsigned char *inbuf,	/* Data received from socket */
@@ -164,6 +168,45 @@ struct TELNET {
   TelnetReceive telrcv_state;
 };
 
+#ifdef WIN32
+static CURLcode
+check_wsock2 ( struct SessionHandle *data )
+{
+  int err; 
+  WORD wVersionRequested;  
+  WSADATA wsaData; 
+
+  curlassert(data);
+
+  /* telnet requires at least WinSock 2.0 so ask for it. */
+  wVersionRequested = MAKEWORD(2, 0);
+
+  err = WSAStartup(wVersionRequested, &wsaData); 
+  
+  /* We must've called this once already, so this call */
+  /* should always succeed.  But, just in case... */
+  if (err != 0) {
+    failf(data,"WSAStartup failed (%d)",err);
+    return CURLE_FAILED_INIT; 
+  }
+
+  /* We have to have a WSACleanup call for every successful */
+  /* WSAStartup call. */
+  WSACleanup();
+
+  /* Check that our version is supported */
+  if (LOBYTE(wsaData.wVersion) != LOBYTE(wVersionRequested) ||
+      HIBYTE(wsaData.wVersion) != HIBYTE(wVersionRequested)) {
+      /* Our version isn't supported */
+      failf(data,"insufficient winsock version to support "
+	    "telnet");
+      return CURLE_FAILED_INIT;
+  }
+
+  /* Our version is supported */
+  return CURLE_OK;
+}
+#endif
 static
 CURLcode init_telnet(struct connectdata *conn)
 {
@@ -263,7 +306,7 @@ static void send_negotiation(struct connectdata *conn, int cmd, int option)
    buf[1] = cmd;
    buf[2] = option;
    
-   (void)swrite(conn->firstsocket, buf, 3);
+   (void)swrite(conn->sock[FIRSTSOCKET], buf, 3);
    
    printoption(conn->data, "SENT", cmd, option);
 }
@@ -741,7 +784,7 @@ static void printsub(struct SessionHandle *data,
   }
 }
 
-static int check_telnet_options(struct connectdata *conn)
+static CURLcode check_telnet_options(struct connectdata *conn)
 {
   struct curl_slist *head;
   char option_keyword[128];
@@ -754,9 +797,8 @@ static int check_telnet_options(struct connectdata *conn)
      was given on the command line */
   if(conn->bits.user_passwd)
   {
-    char *buf = malloc(256);
-    sprintf(buf, "USER,%s", conn->user);
-    tn->telnet_vars = curl_slist_append(tn->telnet_vars, buf);
+    snprintf(option_arg, sizeof(option_arg), "USER,%s", conn->user);
+    tn->telnet_vars = curl_slist_append(tn->telnet_vars, option_arg);
 
     tn->us_preferred[CURL_TELOPT_NEW_ENVIRON] = CURL_YES;
   }
@@ -812,23 +854,22 @@ static int check_telnet_options(struct connectdata *conn)
 static void suboption(struct connectdata *conn)
 {
   struct curl_slist *v;
-  unsigned char subchar;
   unsigned char temp[2048];
-  int len;
-  int tmplen;
+  size_t len;
+  size_t tmplen;
   char varname[128];
   char varval[128];
   struct SessionHandle *data = conn->data;
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
 
   printsub(data, '<', (unsigned char *)tn->subbuffer, CURL_SB_LEN(tn)+2);
-  switch (subchar = CURL_SB_GET(tn)) {
+  switch (CURL_SB_GET(tn)) {
     case CURL_TELOPT_TTYPE:
       len = strlen(tn->subopt_ttype) + 4 + 2;
       snprintf((char *)temp, sizeof(temp),
                "%c%c%c%c%s%c%c", CURL_IAC, CURL_SB, CURL_TELOPT_TTYPE,
                CURL_TELQUAL_IS, tn->subopt_ttype, CURL_IAC, CURL_SE);
-      (void)swrite(conn->firstsocket, temp, len);
+      (void)swrite(conn->sock[FIRSTSOCKET], temp, len);
       printsub(data, '>', &temp[2], len-2);
       break;
     case CURL_TELOPT_XDISPLOC:
@@ -836,7 +877,7 @@ static void suboption(struct connectdata *conn)
       snprintf((char *)temp, sizeof(temp),
                "%c%c%c%c%s%c%c", CURL_IAC, CURL_SB, CURL_TELOPT_XDISPLOC,
                CURL_TELQUAL_IS, tn->subopt_xdisploc, CURL_IAC, CURL_SE);
-      (void)swrite(conn->firstsocket, temp, len);
+      (void)swrite(conn->sock[FIRSTSOCKET], temp, len);
       printsub(data, '>', &temp[2], len-2);
       break;
     case CURL_TELOPT_NEW_ENVIRON:
@@ -859,7 +900,7 @@ static void suboption(struct connectdata *conn)
       snprintf((char *)&temp[len], sizeof(temp) - len,
                "%c%c", CURL_IAC, CURL_SE);
       len += 2;
-      (void)swrite(conn->firstsocket, temp, len);
+      (void)swrite(conn->sock[FIRSTSOCKET], temp, len);
       printsub(data, '>', &temp[2], len-2);
       break;
   }
@@ -872,13 +913,13 @@ void telrcv(struct connectdata *conn,
             int count)			/* Number of bytes received */
 {
   unsigned char c;
-  int index = 0;
+  int in = 0;
   struct SessionHandle *data = conn->data;
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
 
   while(count--)
   {
-    c = inbuf[index++];
+    c = inbuf[in++];
 
     switch (tn->telrcv_state)
     {
@@ -1022,9 +1063,11 @@ void telrcv(struct connectdata *conn,
   }
 }
 
-CURLcode Curl_telnet_done(struct connectdata *conn)
+CURLcode Curl_telnet_done(struct connectdata *conn, CURLcode status)
 {
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  (void)status; /* unused */
+
   curl_slist_free_all(tn->telnet_vars);
 
   free(conn->proto.telnet);
@@ -1037,20 +1080,28 @@ CURLcode Curl_telnet(struct connectdata *conn)
 {
   CURLcode code;
   struct SessionHandle *data = conn->data;
-  int sockfd = conn->firstsocket;
+  curl_socket_t sockfd = conn->sock[FIRSTSOCKET];
 #ifdef WIN32
+  HMODULE wsock2;
+  WSOCK2_FUNC close_event_func;
+  WSOCK2_FUNC create_event_func;
+  WSOCK2_FUNC event_select_func;
+  WSOCK2_FUNC enum_netevents_func;
   WSAEVENT event_handle;
   WSANETWORKEVENTS events;
   HANDLE stdin_handle;
-  HANDLE objs[2];
+  HANDLE objs[2];         
+  DWORD  obj_count;
+  DWORD  wait_timeout;
   DWORD waitret;
+  DWORD readfile_read;
 #else
   fd_set readfd;
   fd_set keepfd;
 #endif
+  ssize_t nread;  
   bool keepon = TRUE;
   char *buf = data->state.buffer;
-  ssize_t nread;
   struct TELNET *tn;
 
   code = init_telnet(conn);
@@ -1064,85 +1115,205 @@ CURLcode Curl_telnet(struct connectdata *conn)
     return code;
 
 #ifdef WIN32
+  /*
+  ** This functionality only works with WinSock >= 2.0.  So,
+  ** make sure have it.
+  */
+  code = check_wsock2(data);
+  if (code)
+    return code;
+
+  /* OK, so we have WinSock 2.0.  We need to dynamically */
+  /* load ws2_32.dll and get the function pointers we need. */
+  wsock2 = LoadLibrary("WS2_32.DLL");
+  if (wsock2 == NULL) {
+    failf(data,"failed to load WS2_32.DLL (%d)",GetLastError());
+    return CURLE_FAILED_INIT;
+  }
+
+  /* Grab a pointer to WSACreateEvent */
+  create_event_func = GetProcAddress(wsock2,"WSACreateEvent");
+  if (create_event_func == NULL) {
+    failf(data,"failed to find WSACreateEvent function (%d)",
+	  GetLastError());
+    FreeLibrary(wsock2);
+    return CURLE_FAILED_INIT;
+  }
+
+  /* And WSACloseEvent */
+  close_event_func = GetProcAddress(wsock2,"WSACloseEvent");
+  if (create_event_func == NULL) {
+    failf(data,"failed to find WSACloseEvent function (%d)",
+	  GetLastError());
+    FreeLibrary(wsock2);
+    return CURLE_FAILED_INIT;
+  }
+
+  /* And WSAEventSelect */
+  event_select_func = GetProcAddress(wsock2,"WSAEventSelect");
+  if (event_select_func == NULL) {
+    failf(data,"failed to find WSAEventSelect function (%d)",
+	  GetLastError());
+    FreeLibrary(wsock2);
+    return CURLE_FAILED_INIT;
+  }
+
+  /* And WSAEnumNetworkEvents */
+  enum_netevents_func = GetProcAddress(wsock2,"WSAEnumNetworkEvents");
+  if (enum_netevents_func == NULL) {
+    failf(data,"failed to find WSAEnumNetworkEvents function (%d)",
+	  GetLastError());
+    FreeLibrary(wsock2);
+    return CURLE_FAILED_INIT;
+  }
+
   /* We want to wait for both stdin and the socket. Since
   ** the select() function in winsock only works on sockets
   ** we have to use the WaitForMultipleObjects() call.
   */
 
   /* First, create a sockets event object */
-  event_handle = WSACreateEvent();
+  event_handle = (WSAEVENT)create_event_func();
+  if (event_handle == WSA_INVALID_EVENT) {
+    failf(data,"WSACreateEvent failed (%d)",WSAGetLastError());
+    FreeLibrary(wsock2);
+    return CURLE_FAILED_INIT;
+  }
 
   /* The get the Windows file handle for stdin */
   stdin_handle = GetStdHandle(STD_INPUT_HANDLE);
 
   /* Create the list of objects to wait for */
-  objs[0] = stdin_handle;
-  objs[1] = event_handle;
+  objs[0] = event_handle;
+  objs[1] = stdin_handle;
 
   /* Tell winsock what events we want to listen to */
-  if(WSAEventSelect(sockfd, event_handle, FD_READ|FD_CLOSE) == SOCKET_ERROR) {
+  if(event_select_func(sockfd, event_handle, FD_READ|FD_CLOSE) == SOCKET_ERROR) {
+    close_event_func(event_handle);
+    FreeLibrary(wsock2);
     return 0;
   }
-
+                                   
+  /* If stdin_handle is a pipe, use PeekNamedPipe() method to check it, 
+     else use the old WaitForMultipleObjects() way */         
+  if(GetFileType(stdin_handle) == FILE_TYPE_PIPE) {
+    /* Don't wait for stdin_handle, just wait for event_handle */
+    obj_count = 1;      
+    /* Check stdin_handle per 100 milliseconds */
+    wait_timeout = 100;  
+  } else {
+    obj_count = 2;
+    wait_timeout = INFINITE;
+  }
+  
   /* Keep on listening and act on events */
   while(keepon) {
-    waitret = WaitForMultipleObjects(2, objs, FALSE, INFINITE);
-    switch(waitret - WAIT_OBJECT_0)
+    waitret = WaitForMultipleObjects(obj_count, objs, FALSE, wait_timeout);
+    switch(waitret) {
+    case WAIT_TIMEOUT:
     {
-      case 0:
-      {
-        unsigned char outbuf[2];
-        int out_count = 0;
-        ssize_t bytes_written;
-        char *buffer = buf;
-              
-        if(!ReadFile(stdin_handle, buf, 255, (LPDWORD)&nread, NULL)) {
+      unsigned char outbuf[2];
+      int out_count = 0;
+      ssize_t bytes_written;
+      char *buffer = buf;
+
+      for(;;) {
+        if(!PeekNamedPipe(stdin_handle,NULL,0,NULL,&nread,NULL)) {
           keepon = FALSE;
           break;
         }
         
+        if(!nread)
+          break;
+          
+        if(!ReadFile(stdin_handle, buf, sizeof(data->state.buffer),
+                     &readfile_read, NULL)) {
+          keepon = FALSE;
+          break;
+        }
+        nread = readfile_read;
+        
         while(nread--) {
           outbuf[0] = *buffer++;
           out_count = 1;
           if(outbuf[0] == CURL_IAC)
             outbuf[out_count++] = CURL_IAC;
-          
-          Curl_write(conn, conn->firstsocket, outbuf,
+            
+          Curl_write(conn, conn->sock[FIRSTSOCKET], outbuf,
                      out_count, &bytes_written);
         }
       }
-      break;
-      
-      case 1:
-        if(WSAEnumNetworkEvents(sockfd, event_handle, &events)
-           != SOCKET_ERROR)
-        {
-          if(events.lNetworkEvents & FD_READ)
-          {
-            /* This reallu OUGHT to check its return code. */
-            Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
-            
-            telrcv(conn, (unsigned char *)buf, nread);
-            
-            fflush(stdout);
-            
-            /* Negotiate if the peer has started negotiating,
-               otherwise don't. We don't want to speak telnet with
-               non-telnet servers, like POP or SMTP. */
-            if(tn->please_negotiate && !tn->already_negotiated) {
-              negotiate(conn);
-              tn->already_negotiated = 1;
-            }
-          }
+    }      
+    break;
+    
+    case WAIT_OBJECT_0 + 1:
+    {
+      unsigned char outbuf[2];
+      int out_count = 0;
+      ssize_t bytes_written;
+      char *buffer = buf;
+              
+      if(!ReadFile(stdin_handle, buf, sizeof(data->state.buffer),
+                   &readfile_read, NULL)) {
+        keepon = FALSE;
+        break;
+      }
+      nread = readfile_read;
+        
+      while(nread--) {
+        outbuf[0] = *buffer++;
+        out_count = 1;
+        if(outbuf[0] == CURL_IAC)
+          outbuf[out_count++] = CURL_IAC;
           
-          if(events.lNetworkEvents & FD_CLOSE)
-          {
-            keepon = FALSE;
+        Curl_write(conn, conn->sock[FIRSTSOCKET], outbuf,
+                   out_count, &bytes_written);
+      }
+    }
+    break;
+      
+    case WAIT_OBJECT_0:
+      if(enum_netevents_func(sockfd, event_handle, &events)
+         != SOCKET_ERROR) {
+        if(events.lNetworkEvents & FD_READ) {
+          /* This reallu OUGHT to check its return code. */
+          (void)Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
+            
+          telrcv(conn, (unsigned char *)buf, nread);
+          
+          fflush(stdout);
+            
+          /* Negotiate if the peer has started negotiating,
+             otherwise don't. We don't want to speak telnet with
+             non-telnet servers, like POP or SMTP. */
+          if(tn->please_negotiate && !tn->already_negotiated) {
+            negotiate(conn);
+            tn->already_negotiated = 1;
           }
         }
-        break;
+          
+        if(events.lNetworkEvents & FD_CLOSE) {
+          keepon = FALSE;
+        }
+      }
+      break;
     }
   }
+
+  /* We called WSACreateEvent, so call WSACloseEvent */
+  if (close_event_func(event_handle) == FALSE) {
+    infof(data,"WSACloseEvent failed (%d)",WSAGetLastError());
+  }
+
+  /* "Forget" pointers into the library we're about to free */
+  create_event_func = NULL;
+  close_event_func = NULL;
+  event_select_func = NULL;
+  enum_netevents_func = NULL;
+
+  /* We called LoadLibrary, so call FreeLibrary */
+  if (!FreeLibrary(wsock2))
+    infof(data,"FreeLibrary(wsock2) failed (%d)",GetLastError());
 #else
   FD_ZERO (&readfd);		/* clear it */
   FD_SET (sockfd, &readfd);
@@ -1178,14 +1349,14 @@ CURLcode Curl_telnet(struct connectdata *conn)
           if(outbuf[0] == CURL_IAC)
             outbuf[out_count++] = CURL_IAC;
       
-          Curl_write(conn, conn->firstsocket, outbuf,
+          Curl_write(conn, conn->sock[FIRSTSOCKET], outbuf,
                      out_count, &bytes_written);
         }
       }
 
       if(FD_ISSET(sockfd, &readfd)) {
         /* This OUGHT to check the return code... */
-        Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
+        (void)Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
 
         /* if we receive 0 or less here, the server closed the connection and
            we bail out from this! */
