@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: version.c,v 1.26 2003/08/11 23:13:09 bagder Exp $
+ * $Id: version.c,v 1.39 2004/05/24 07:40:01 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -29,6 +29,14 @@
 #include <curl/curl.h>
 #include "urldata.h"
 
+#ifdef USE_ARES
+#include <ares_version.h>
+#endif
+
+#ifdef USE_LIBIDN
+#include <stringprep.h>
+#endif
+
 #ifdef USE_SSLEAY
 static void getssl_version(char *ptr, long *num)
 {
@@ -39,14 +47,14 @@ static void getssl_version(char *ptr, long *num)
     unsigned long ssleay_value;
     sub[1]='\0';
     ssleay_value=SSLeay();
-    *num = ssleay_value;
+    *num = (long)ssleay_value;
     if(ssleay_value < 0x906000) {
       ssleay_value=SSLEAY_VERSION_NUMBER;
       sub[0]='\0';
     }
     else {
       if(ssleay_value&0xff0) {
-        sub[0]=((ssleay_value>>4)&0xff) + 'a' -1;
+        sub[0]=(char)((ssleay_value>>4)&0xff) + 'a' -1;
       }
       else
         sub[0]='\0';
@@ -90,19 +98,19 @@ static void getssl_version(char *ptr, long *num)
 char *curl_version(void)
 {
   static char version[200];
-  char *ptr;
-  long num;
-  strcpy(version, LIBCURL_NAME "/" LIBCURL_VERSION );
-  ptr=strchr(version, '\0');
+  char *ptr=version;
+  strcpy(ptr, LIBCURL_NAME "/" LIBCURL_VERSION );
+  ptr=strchr(ptr, '\0');
 
 #ifdef USE_SSLEAY
-  getssl_version(ptr, &num);
-  ptr=strchr(version, '\0');
-#else
-  (void)num; /* no compiler warning please */
+  {
+    long num;
+    getssl_version(ptr, &num);
+    ptr=strchr(version, '\0');
+  }
 #endif
 
-#ifdef KRB4
+#ifdef HAVE_KRB4
   sprintf(ptr, " krb4");
   ptr += strlen(ptr);
 #endif
@@ -114,10 +122,21 @@ char *curl_version(void)
   sprintf(ptr, " zlib/%s", zlibVersion());
   ptr += strlen(ptr);
 #endif
-#ifdef GSSAPI
+#ifdef HAVE_GSSAPI
   sprintf(ptr, " GSS");
   ptr += strlen(ptr);
 #endif
+#ifdef USE_ARES
+  /* this function is only present in c-ares, not in the original ares */
+  sprintf(ptr, " c-ares/%s", ares_version(NULL));
+  ptr += strlen(ptr);
+#endif
+#ifdef USE_LIBIDN
+  if(stringprep_check_version(LIBIDN_REQUIRED_VERSION)) {
+    sprintf(ptr, " libidn/%s", stringprep_check_version(NULL));
+    ptr += strlen(ptr);
+  }
+#endif
 
   return version;
 }
@@ -159,7 +178,7 @@ static const char *protocols[] = {
 };
 
 static curl_version_info_data version_info = {
-  CURLVERSION_FIRST,
+  CURLVERSION_NOW,
   LIBCURL_VERSION,
   LIBCURL_VERSION_NUM,
   OS, /* as found by configure or set by hand at build-time */
@@ -167,7 +186,7 @@ static curl_version_info_data version_info = {
 #ifdef ENABLE_IPV6
   | CURL_VERSION_IPV6
 #endif
-#ifdef KRB4
+#ifdef HAVE_KRB4
   | CURL_VERSION_KERBEROS4
 #endif
 #ifdef USE_SSLEAY
@@ -177,7 +196,7 @@ static curl_version_info_data version_info = {
 #ifdef HAVE_LIBZ
   | CURL_VERSION_LIBZ
 #endif
-#ifdef GSSAPI
+#ifdef HAVE_GSSAPI
   | CURL_VERSION_GSSNEGOTIATE
 #endif
 #ifdef CURLDEBUG
@@ -185,12 +204,21 @@ static curl_version_info_data version_info = {
 #endif
 #ifdef USE_ARES
   | CURL_VERSION_ASYNCHDNS
+#endif
+#ifdef HAVE_SPNEGO
+  | CURL_VERSION_SPNEGO
+#endif
+#if defined(ENABLE_64BIT) && (SIZEOF_CURL_OFF_T > 4)
+  | CURL_VERSION_LARGEFILE
 #endif
   ,
   NULL, /* ssl_version */
   0,    /* ssl_version_num */
   NULL, /* zlib_version */
-  protocols
+  protocols,
+  NULL, /* c-ares version */
+  0,    /* c-ares version numerical */
+  NULL, /* libidn version */
 };
 
 curl_version_info_data *curl_version_info(CURLversion stamp)
@@ -208,6 +236,20 @@ curl_version_info_data *curl_version_info(CURLversion stamp)
 #ifdef HAVE_LIBZ
   version_info.libz_version = zlibVersion();
   /* libz left NULL if non-existing */
+#endif
+#ifdef USE_ARES
+  {
+    int aresnum;
+    version_info.ares = ares_version(&aresnum);
+    version_info.ares_num = aresnum;
+  }
+#endif
+#ifdef USE_LIBIDN
+  /* This returns a version string if we use the given version or later,
+     otherwise it returns NULL */
+  version_info.libidn = stringprep_check_version(LIBIDN_REQUIRED_VERSION);
+  if(version_info.libidn)
+    version_info.features |= CURL_VERSION_IDN;
 #endif
   (void)stamp; /* avoid compiler warnings, we don't use this */
 
