@@ -7,7 +7,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -20,25 +20,27 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.20 2003/07/29 11:07:38 bagder Exp $
+ * $Id: setup.h,v 1.30 2004/03/21 22:46:07 bagder Exp $
  ***************************************************************************/
 
+#define CURL_NO_OLDIES
+
 #if !defined(WIN32) && defined(__WIN32__)
 /* Borland fix */
 #define WIN32
 #endif
 
 #ifdef HAVE_CONFIG_H
-#ifdef VMS
-#include "config-vms.h"
-#else
 #include "config.h" /* the configure script results */
-#endif
 #else
 #ifdef WIN32
 /* include the hand-modified win32 adjusted config.h! */
 #include "config-win32.h"
 #endif
+#ifdef NETWARE
+/* hand-modified NetWare config.h! */
+#include "config-netware.h"
+#endif
 #ifdef macintosh
 /* this is not the same as Mac OS X */
 #include "config-mac.h"
@@ -46,6 +48,9 @@
 #ifdef __riscos__
 #include "config-riscos.h"
 #endif
+#ifdef __amigaos__
+#include "config-amigaos.h"
+#endif
 #endif
 
 #ifdef CURLDEBUG
@@ -57,6 +62,17 @@
 
 #include <stdio.h>
 
+#ifdef __TANDEM
+#include <floss.h>
+#endif
+
+#ifndef TRUE
+#define TRUE 1
+#endif
+#ifndef FALSE
+#define FALSE 0
+#endif
+
 #ifndef OS
 #define OS "unknown"
 #endif
@@ -82,6 +98,7 @@ int fileno( FILE *stream);
 #ifdef word
 #undef word
 #endif
+#define HAVE_LIMITS_H /* we have limits.h */
 #define DIR_CHAR      "/"
 #define DOT_CHAR      "_"
 #else
@@ -97,4 +114,12 @@ int fileno( FILE *stream);
 #define USE_ENVIRONMENT
 #endif
 
+#ifdef __BEOS__
+#define typedef_bool
+#endif
+
+#ifndef SIZEOF_CURL_OFF_T
+#define SIZEOF_CURL_OFF_T sizeof(curl_off_t)
+#endif
+
 #endif /* __SETUP_H */
