@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: urlglob.c,v 1.26 2003/08/14 15:01:20 bagder Exp $
+ * $Id: urlglob.c,v 1.34 2004/05/12 08:22:04 bagder Exp $
  ***************************************************************************/
 
 /* client-local setup.h */
@@ -35,12 +35,10 @@
 
 #include "urlglob.h"
 
-
-#ifdef CURLDEBUG
-#include "../lib/memdebug.h"
+#if defined(CURLDEBUG) && defined(CURLTOOLDEBUG)
+#include "memdebug.h"
 #endif
 
-char globerrormsg[80]; /* nasty global error message buffer for globbing */
 typedef enum {
   GLOB_OK,
   GLOB_ERROR
@@ -54,10 +52,11 @@ typedef enum {
  */
 static GlobCode glob_word(URLGlob *, /* object anchor */
                           char *,    /* globbed string */
-                          int,       /* position */
+                          size_t,       /* position */
                           int *);    /* returned number of strings */
 
-static GlobCode glob_set(URLGlob *glob, char *pattern, int pos, int *amount)
+static GlobCode glob_set(URLGlob *glob, char *pattern,
+                         size_t pos, int *amount)
 {
   /* processes a set expression with the point behind the opening '{'
      ','-separated elements are collected until the next closing '}'
@@ -76,14 +75,14 @@ static GlobCode glob_set(URLGlob *glob, char *pattern, int pos, int *amount)
   while (1) {
     switch (*pattern) {
     case '\0':			/* URL ended while set was still open */
-      snprintf(globerrormsg, sizeof(globerrormsg),
-               "unmatched brace at pos %d\n", pos);
+      snprintf(glob->errormsg, sizeof(glob->errormsg),
+               "unmatched brace at pos %d\n", (int)pos);
       return GLOB_ERROR;
 
     case '{':
     case '[':			/* no nested expressions at this time */
-      snprintf(globerrormsg, sizeof(globerrormsg),
-               "nested braces not supported at pos %d\n", pos);
+      snprintf(glob->errormsg, sizeof(glob->errormsg),
+               "nested braces not supported at pos %d\n", (int)pos);
       return GLOB_ERROR;
 
     case ',':
@@ -93,7 +92,7 @@ static GlobCode glob_set(URLGlob *glob, char *pattern, int pos, int *amount)
         realloc(pat->content.Set.elements,
                 (pat->content.Set.size + 1) * sizeof(char*));
       if (!pat->content.Set.elements) {
-        snprintf(globerrormsg, sizeof(globerrormsg), "out of memory");
+        snprintf(glob->errormsg, sizeof(glob->errormsg), "out of memory");
         return GLOB_ERROR;
       }
       pat->content.Set.elements[pat->content.Set.size] =
@@ -118,14 +117,14 @@ static GlobCode glob_set(URLGlob *glob, char *pattern, int pos, int *amount)
       break;
 
     case ']':				/* illegal closing bracket */
-      snprintf(globerrormsg, sizeof(globerrormsg), 
-               "illegal pattern at pos %d\n", pos);
+      snprintf(glob->errormsg, sizeof(glob->errormsg), 
+               "illegal pattern at pos %d\n", (int)pos);
       return GLOB_ERROR;
 
     case '\\':				/* escaped character, skip '\' */
       if (*(buf+1) == '\0') {		/* but no escaping of '\0'! */
-        snprintf(globerrormsg, sizeof(globerrormsg), 
-                 "illegal pattern at pos %d\n", pos);
+        snprintf(glob->errormsg, sizeof(glob->errormsg), 
+                 "illegal pattern at pos %d\n", (int)pos);
 	return GLOB_ERROR;
       }
       ++pattern;
@@ -136,8 +135,7 @@ static GlobCode glob_set(URLGlob *glob, char *pattern, int pos, int *amount)
       ++pos;
     }
   }
-  snprintf(globerrormsg, sizeof(globerrormsg), "malformatted pattern");
-  return GLOB_ERROR;
+  /* we never reach this point */
 }
 
 static GlobCode glob_range(URLGlob *glob, char *pattern, int pos, int *amount)
@@ -163,7 +161,7 @@ static GlobCode glob_range(URLGlob *glob, char *pattern, int pos, int *amount)
 	pat->content.CharRange.min_c >= pat->content.CharRange.max_c ||
 	pat->content.CharRange.max_c - pat->content.CharRange.min_c > 'z' - 'a') {
       /* the pattern is not well-formed */ 
-      snprintf(globerrormsg, sizeof(globerrormsg),
+      snprintf(glob->errormsg, sizeof(glob->errormsg),
                "illegal pattern or range specification after pos %d\n", pos);
       return GLOB_ERROR;
     }
@@ -189,7 +187,7 @@ static GlobCode glob_range(URLGlob *glob, char *pattern, int pos, int *amount)
                &pat->content.NumRange.max_n) != 2 ||
 	pat->content.NumRange.min_n >= pat->content.NumRange.max_n) {
       /* the pattern is not well-formed */ 
-      snprintf(globerrormsg, sizeof(globerrormsg), 
+      snprintf(glob->errormsg, sizeof(glob->errormsg), 
                "error: illegal pattern or range specification after pos %d\n",
                pos);
       return GLOB_ERROR;
@@ -205,7 +203,7 @@ static GlobCode glob_range(URLGlob *glob, char *pattern, int pos, int *amount)
     if(c)
       c++;
     else {
-      snprintf(globerrormsg, sizeof(globerrormsg), "missing ']'");
+      snprintf(glob->errormsg, sizeof(glob->errormsg), "missing ']'");
       return GLOB_ERROR; /* missing ']' */
     }
 
@@ -220,18 +218,19 @@ static GlobCode glob_range(URLGlob *glob, char *pattern, int pos, int *amount)
 
     return GLOB_OK;
   }
-  snprintf(globerrormsg, sizeof(globerrormsg), 
+  snprintf(glob->errormsg, sizeof(glob->errormsg), 
            "illegal character in range specification at pos %d\n", pos);
   return GLOB_ERROR;
 }
 
-static GlobCode glob_word(URLGlob *glob, char *pattern, int pos, int *amount)
+static GlobCode glob_word(URLGlob *glob, char *pattern,
+                          size_t pos, int *amount)
 {
   /* processes a literal string component of a URL
      special characters '{' and '[' branch to set/range processing functions
    */ 
   char* buf = glob->glob_buffer;
-  int litindex;
+  size_t litindex;
   GlobCode res = GLOB_OK;
 
   *amount = 1; /* default is one single string */
@@ -294,15 +293,13 @@ int glob_url(URLGlob** glob, char* url, int *urlnum, FILE *error)
   int amount;
   char *glob_buffer=(char *)malloc(strlen(url)+1);
 
-  if(NULL == glob_buffer) {
-    snprintf(globerrormsg, sizeof(globerrormsg), "out of memory");
+  *glob = NULL;
+  if(NULL == glob_buffer)
     return CURLE_OUT_OF_MEMORY;
-  }
 
   glob_expand = (URLGlob*)malloc(sizeof(URLGlob));
   if(NULL == glob_expand) {
     free(glob_buffer);
-    snprintf(globerrormsg, sizeof(globerrormsg), "out of memory");
     return CURLE_OUT_OF_MEMORY;
   }
   glob_expand->size = 0;
@@ -312,16 +309,16 @@ int glob_url(URLGlob** glob, char* url, int *urlnum, FILE *error)
   if(GLOB_OK == glob_word(glob_expand, url, 1, &amount))
     *urlnum = amount;
   else {
+    if(error && glob_expand->errormsg[0]) {
+      /* send error description to the error-stream */
+      fprintf(error, "curl: (%d) [globbing] %s\n",
+              CURLE_URL_MALFORMAT, glob_expand->errormsg);
+    }
     /* it failed, we cleanup */
     free(glob_buffer);
     free(glob_expand);
     glob_expand = NULL;
     *urlnum = 1;
-    if(error && globerrormsg[0]) {
-      /* send error description to the error-stream */
-      fprintf(error, "curl: (%d) [globbing] %s\n",
-              CURLE_URL_MALFORMAT, globerrormsg);
-    }
     return CURLE_URL_MALFORMAT;
   }
 
@@ -357,7 +354,8 @@ char *glob_next_url(URLGlob *glob)
   char *buf = glob->glob_buffer;
   URLPattern *pat;
   char *lit;
-  signed int i;
+  int i;
+  size_t j;
   int carry;
 
   if (!glob->beenhere)
@@ -390,7 +388,7 @@ char *glob_next_url(URLGlob *glob)
 	}
 	break;
       default:
-	printf("internal error: invalid pattern type (%d)\n", pat->type);
+	printf("internal error: invalid pattern type (%d)\n", (int)pat->type);
 	exit (CURLE_FAILED_INIT);
       }
     }
@@ -398,14 +396,14 @@ char *glob_next_url(URLGlob *glob)
       return NULL;
   }
 
-  for (i = 0; i < glob->size; ++i) {
-    if (!(i % 2)) {            /* every other term (i even) is a literal */
-      lit = glob->literal[i/2];
+  for (j = 0; j < glob->size; ++j) {
+    if (!(j&1)) {              /* every other term (j even) is a literal */
+      lit = glob->literal[j/2];
       strcpy(buf, lit);
       buf += strlen(lit);
     }
     else {				/* the rest (i odd) are patterns */
-      pat = &glob->pattern[i/2];
+      pat = &glob->pattern[j/2];
       switch(pat->type) {
       case UPTSet:
 	strcpy(buf, pat->content.Set.elements[pat->content.Set.ptr_s]);
@@ -420,7 +418,7 @@ char *glob_next_url(URLGlob *glob)
         buf += strlen(buf); /* make no sprint() return code assumptions */
 	break;
       default:
-	printf("internal error: invalid pattern type (%d)\n", pat->type);
+	printf("internal error: invalid pattern type (%d)\n", (int)pat->type);
 	exit (CURLE_FAILED_INIT);
       }
     }
@@ -432,11 +430,11 @@ char *glob_next_url(URLGlob *glob)
 char *glob_match_url(char *filename, URLGlob *glob)
 {
   char *target;
-  int allocsize;
-  int stringlen=0;
+  size_t allocsize;
+  size_t stringlen=0;
   char numbuf[18];
-  char *appendthis;
-  int appendlen;
+  char *appendthis = NULL;
+  size_t appendlen = 0;
 
   /* We cannot use the glob_buffer for storage here since the filename may
    * be longer than the URL we use. We allocate a good start size, then
@@ -450,7 +448,7 @@ char *glob_match_url(char *filename, URLGlob *glob)
   while (*filename) {
     if (*filename == '#' && isdigit((int)filename[1])) {
       /* only '#1' ... '#9' allowed */
-      int i;
+      unsigned long i;
       unsigned long num = strtoul(&filename[1], &filename, 10);
 
       i = num-1;
@@ -460,8 +458,7 @@ char *glob_match_url(char *filename, URLGlob *glob)
         switch (pat.type) {
         case UPTSet:
           appendthis = pat.content.Set.elements[pat.content.Set.ptr_s];
-          appendlen =
-            (int)strlen(pat.content.Set.elements[pat.content.Set.ptr_s]);
+          appendlen = strlen(pat.content.Set.elements[pat.content.Set.ptr_s]);
           break;
         case UPTCharRange:
           numbuf[0]=pat.content.CharRange.ptr_c;
@@ -474,10 +471,11 @@ char *glob_match_url(char *filename, URLGlob *glob)
                   pat.content.NumRange.padlength,
                   pat.content.NumRange.ptr_n);
           appendthis = numbuf;
-          appendlen = (int)strlen(numbuf);
+          appendlen = strlen(numbuf);
           break;
         default:
-          printf("internal error: invalid pattern type (%d)\n", pat.type);
+          printf("internal error: invalid pattern type (%d)\n",
+                 (int)pat.type);
           free(target);
           return NULL;
         }
