@@ -1,6 +1,27 @@
 #!/usr/bin/perl
+#***************************************************************************
+#                                  _   _ ____  _
+#  Project                     ___| | | |  _ \| |
+#                             / __| | | | |_) | |
+#                            | (__| |_| |  _ <| |___
+#                             \___|\___/|_| \_\_____|
 #
-# $Id: ftpserver.pl,v 1.36 2003/08/08 10:21:47 bagder Exp $
+# Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://curl.haxx.se/docs/copyright.html.
+#
+# You may opt to use, copy, modify, merge, publish, distribute and/or sell
+# copies of the Software, and permit persons to whom the Software is
+# furnished to do so, under the terms of the COPYING file.
+#
+# This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+# KIND, either express or implied.
+#
+# $Id: ftpserver.pl,v 1.43 2004/05/17 08:02:23 bagder Exp $
+###########################################################################
+
 # This is the FTP server designed for the curl test suite.
 #
 # It is meant to exercise curl, it is not meant to be a fully working
@@ -11,17 +32,22 @@
 #
 
 use Socket;
-use Carp;
 use FileHandle;
 
 use strict;
 
 require "getpart.pm";
 
-open(FTPLOG, ">log/ftpd.log") ||
-    print STDERR "failed to open log file, runs without logging\n";
-
-sub logmsg { print FTPLOG "$$: "; print FTPLOG @_; }
+# open and close each time to allow removal at any time
+sub logmsg {
+    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
+        localtime(time);
+    open(FTPLOG, ">>log/ftpd.log");
+    printf FTPLOG ("%02d:%02d:%02d (%d) ",
+                   $hour, $min, $sec, $$);
+    print FTPLOG @_;
+    close(FTPLOG);
+}
 
 sub ftpmsg {
   # append to the server.input file
@@ -40,12 +66,17 @@ sub ftpmsg {
 my $verbose=0; # set to 1 for debugging
 my $retrweirdo=0;
 my $retrnosize=0;
+my $srcdir=".";
 
 my $port = 8921; # just a default
 do {
     if($ARGV[0] eq "-v") {
         $verbose=1;
     }
+    elsif($ARGV[0] eq "-s") {
+        $srcdir=$ARGV[1];
+        shift @ARGV;
+    }
     elsif($ARGV[0] =~ /^(\d+)$/) {
         $port = $1;
     }
@@ -140,6 +171,12 @@ my %commandfunc = ( 'PORT' => \&PORT_command,
                     'MDTM' => \&MDTM_command,
                     );
 
+
+sub close_dataconn {
+    close(SOCK);
+    logmsg "Closed data connection\n";
+}
+
 my $rest=0;
 sub REST_command {
     $rest = $_[0];
@@ -162,12 +199,12 @@ my @ftpdir=("total 20\r\n",
 "drwxrwxrwx   2 98       1            512 Oct 30 14:33 pub\r\n",
 "dr-xr-xr-x   5 0        1            512 Oct  1  1997 usr\r\n");
 
-    logmsg "$$: pass data to child pid\n";
+    logmsg "pass LIST data on data connection\n";
     for(@ftpdir) {
         print SOCK $_;
     }
-    close(SOCK);
-    logmsg "$$: done passing data to child pid\n";
+    close_dataconn();
+    logmsg "done passing data\n";
 
     print "226 ASCII transfer complete\r\n";
     return 0;
@@ -175,10 +212,11 @@ my @ftpdir=("total 20\r\n",
 
 sub NLST_command {
     my @ftpdir=("file", "with space", "fake", "..", " ..", "funny", "README");
+    logmsg "pass NLST data on data connection\n";
     for(@ftpdir) {
         print SOCK "$_\r\n";
     }
-    close(SOCK);
+    close_dataconn();
     print "226 ASCII transfer complete\r\n";
     return 0;
 }
@@ -186,7 +224,7 @@ sub NLST_command {
 sub MDTM_command {
     my $testno = $_[0];
 
-    loadtest("data/test$testno");
+    loadtest("$srcdir/data/test$testno");
 
     logmsg "MDTM $testno\n";
 
@@ -213,9 +251,9 @@ sub MDTM_command {
 sub SIZE_command {
     my $testno = $_[0];
 
-    loadtest("data/test$testno");
+    loadtest("$srcdir/data/test$testno");
 
-    logmsg "SIZE number $testno\n";
+    logmsg "SIZE file \"$testno\"\n";
 
     my @data = getpart("reply", "size");
 
@@ -252,14 +290,17 @@ sub SIZE_command {
 sub RETR_command {
     my $testno = $_[0];
 
-    logmsg "RETR test number $testno\n";
+    logmsg "RETR file \"$testno\"\n";
 
     if($testno =~ /^verifiedserver$/) {
         # this is the secret command that verifies that this actually is
         # the curl test server
-        print "150 Binary junk (10 bytes).\r\n";
+        my $response = "WE ROOLZ: $$\r\n";
+        my $len = length($response);
+        print "150 Binary junk ($len bytes).\r\n";
+        logmsg "pass our pid on the data connection\n";
         print SOCK "WE ROOLZ: $$\r\n";
-        close(SOCK);
+        close_dataconn();
         print "226 File transfer complete\r\n";
         if($verbose) {
             print STDERR "FTPD: We returned proof we are the test server\n";
@@ -268,7 +309,7 @@ sub RETR_command {
         return 0;
     }
 
-    loadtest("data/test$testno");
+    loadtest("$srcdir/data/test$testno");
 
     my @data = getpart("reply", "data");
 
@@ -290,11 +331,12 @@ sub RETR_command {
             "226 File transfer complete\r\n";
             logmsg "150+226 in one shot!\n";
 
+            logmsg "pass RETR data on data connection\n";
             for(@data) {
                 my $send = $_;
                 print SOCK $send;
             }
-            close(SOCK);
+            close_dataconn();
             $retrweirdo=0; # switch off the weirdo again!
         }
         else {
@@ -306,12 +348,12 @@ sub RETR_command {
             print "150 Binary data connection for $testno () $sz.\r\n";
             logmsg "150 Binary data connection for $testno () $sz.\n";
 
+            logmsg "pass RETR data on data connection\n";
             for(@data) {
                 my $send = $_;
                 print SOCK $send;
             }
-            close(SOCK);
-
+            close_dataconn();
             print "226 File transfer complete\r\n";
         }
     }
@@ -331,6 +373,8 @@ sub STOR_command {
 
     print "125 Gimme gimme gimme!\r\n";
 
+    logmsg "retrieve STOR data on data connection\n";
+
     open(FILE, ">$filename") ||
         return 0; # failed to open output
 
@@ -341,7 +385,7 @@ sub STOR_command {
         print FILE $line;
     }
     close(FILE);
-    close(SOCK);
+    close_dataconn();
 
     logmsg "received $ulsize bytes upload\n";
 
@@ -389,13 +433,32 @@ sub PASV_command {
         printf("229 Entering Passive Mode (|||%d|)\n", $pasvport);
     }
 
-    my $paddr = accept(SOCK, Server2);
-    my($iport,$iaddr) = sockaddr_in($paddr);
-    my $name = gethostbyaddr($iaddr,AF_INET);
 
-    close(Server2); # close the listener when its served its purpose!
+    my $paddr;
+    eval {
+        local $SIG{ALRM} = sub { die "alarm\n" };
+        alarm 2; # assume swift operations!
+        $paddr = accept(SOCK, Server2);
+        alarm 0;
+    };
+    if ($@) {
+        # timed out
+        
+        close(Server2);
+        logmsg "accept failed\n";
+        return;
+    }
+    else {
+        logmsg "accept worked\n";
 
-    logmsg "$$: data connection from $name [", inet_ntoa($iaddr), "] at port $iport\n";
+        my($iport,$iaddr) = sockaddr_in($paddr);
+        my $name = gethostbyaddr($iaddr,AF_INET);
+
+        close(Server2); # close the listener when its served its purpose!
+
+        logmsg "data connection from $name [", inet_ntoa($iaddr),
+        "] at port $iport\n";
+    }
 
     return;
 }
@@ -486,7 +549,6 @@ for ( $waitedpid = 0;
     open(STDIN,  "<&Client")   || die "can't dup client to stdin";
     open(STDOUT, ">&Client")   || die "can't dup client to stdout";
     
-    FTPLOG->autoflush(1);
     &customize(); # read test control instructions
 
     print @welcome;
@@ -515,7 +577,7 @@ for ( $waitedpid = 0;
         my $FTPARG=$2;
         my $full=$_;
                  
-        logmsg "GOT: ($1) $_\n";
+        logmsg "Received \"$full\"\n";
 
         if($verbose) {
             print STDERR "IN: $full\n";
@@ -532,6 +594,10 @@ for ( $waitedpid = 0;
             # remain in the same state
         }
         else {
+            
+            if($state != $newstate) {
+                logmsg "switch to state $state\n";
+            }
             $state = $newstate;
         }
 
@@ -567,10 +633,7 @@ for ( $waitedpid = 0;
                 \&$func($FTPARG, $FTPCMD);
             }
         }
-
-        logmsg "set to state $state\n";
             
     } # while(1)
     close(Client);
-    close(Client2);
 }
