@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: lib505.c,v 1.2 2003/04/15 14:18:37 bagder Exp $
+ * $Id: lib505.c,v 1.5 2004/05/14 06:27:57 bagder Exp $
  */
 
 #include "test.h"
@@ -34,7 +34,7 @@
  * Example based on source code provided by Erick Nuwendam. Thanks!
  */
 
-CURLcode test(char *URL)
+int test(char *URL)
 {
   CURL *curl;
   CURLcode res;
@@ -72,9 +72,14 @@ CURLcode test(char *URL)
   /* get a curl handle */
   curl = curl_easy_init();
   if(curl) {
+    struct curl_slist *hl;
     /* build a list of commands to pass to libcurl */
-    headerlist = curl_slist_append(headerlist, buf_1);
-    headerlist = curl_slist_append(headerlist, buf_2);
+    hl = curl_slist_append(headerlist, buf_1);
+    if(hl) {
+      headerlist = curl_slist_append(hl, buf_2);
+      if(hl)
+        headerlist = hl;
+    }
 
     /* enable uploading */
     curl_easy_setopt(curl, CURLOPT_UPLOAD, TRUE) ;
@@ -92,8 +97,8 @@ CURLcode test(char *URL)
     curl_easy_setopt(curl, CURLOPT_INFILE, hd_src);
 
     /* and give the size of the upload (optional) */
-    curl_easy_setopt(curl, CURLOPT_INFILESIZE,
-                     (long)file_info.st_size);
+    curl_easy_setopt(curl, CURLOPT_INFILESIZE_LARGE,
+                     file_info.st_size);
 
     /* Now run off and do what you've been told! */
     res = curl_easy_perform(curl);
