@@ -1,9 +1,26 @@
 #!/usr/bin/env perl
-# $Id: runtests.pl,v 1.91 2003/08/14 12:59:54 bagder Exp $
+#***************************************************************************
+#                                  _   _ ____  _
+#  Project                     ___| | | |  _ \| |
+#                             / __| | | | |_) | |
+#                            | (__| |_| |  _ <| |___
+#                             \___|\___/|_| \_\_____|
 #
-# Main curl test script, in perl to run on more platforms
+# Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
 #
-#######################################################################
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://curl.haxx.se/docs/copyright.html.
+#
+# You may opt to use, copy, modify, merge, publish, distribute and/or sell
+# copies of the Software, and permit persons to whom the Software is
+# furnished to do so, under the terms of the COPYING file.
+#
+# This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+# KIND, either express or implied.
+#
+# $Id: runtests.pl,v 1.132 2004/05/17 10:53:34 bagder Exp $
+###########################################################################
 # These should be the only variables that might be needed to get edited:
 
 use strict;
@@ -11,7 +28,6 @@ use strict;
 
 @INC=(@INC, $ENV{'srcdir'}, ".");
 
-require "stunnel.pm"; # stunnel functions
 require "getpart.pm"; # array functions
 
 my $srcdir = $ENV{'srcdir'} || '.';
@@ -23,7 +39,7 @@ my $FTPSPORT=8821;  # this is the FTPS server port
 my $CURL="../src/curl"; # what curl executable to run on the tests
 my $DBGCURL=$CURL; #"../src/.libs/curl";  # alternative for debugging
 my $LOGDIR="log";
-my $TESTDIR="data";
+my $TESTDIR="$srcdir/data";
 my $LIBDIR="./libtest";
 my $SERVERIN="$LOGDIR/server.input"; # what curl sent the server
 my $CURLLOG="$LOGDIR/curl.log"; # all command lines run
@@ -57,9 +73,13 @@ my $memdump="memdump";
 # the path to the script that analyzes the memory debug output file:
 my $memanalyze="./memanalyze.pl";
 
-my $checkstunnel = &checkstunnel;
+my $stunnel = checkcmd("stunnel");
+my $valgrind = checkcmd("valgrind");
+my $gdb = checkcmd("gdb");
 
 my $ssl_version; # set if libcurl is built with SSL support
+my $large_file;  # set if libcurl is built with large file support
+my $has_idn;     # set if libcurl is built with IDN support
 
 my $skipped=0;  # number of tests skipped; reported in main loop
 my %skipped;    # skipped{reason}=counter, reasons for skip
@@ -76,17 +96,50 @@ my $anyway;
 my $gdbthis;      # run test case with gdb debugger
 my $keepoutfiles; # keep stdout and stderr files after tests
 my $listonly;     # only list the tests
+my $postmortem;   # display detailed info about failed tests
 
 my $pwd;          # current working directory
 
 my %run;	  # running server
 
+# torture test variables
+my $torture;
+my $tortnum;
+my $tortalloc;
+
 chomp($pwd = `pwd`);
 
 # enable memory debugging if curl is compiled with it
 $ENV{'CURL_MEMDEBUG'} = 1;
 $ENV{'HOME'}=$pwd;
 
+##########################################################################
+# Clear all possible '*_proxy' environment variables for various protocols
+# to prevent them to interfere with our testing!
+
+my $protocol;
+foreach $protocol (('ftp', 'http', 'ftps', 'https', 'gopher', 'no')) {
+    my $proxy = "${protocol}_proxy";
+    # clear lowercase version
+    $ENV{$proxy}=undef;
+    # clear uppercase version
+    $ENV{uc($proxy)}=undef;
+}
+
+#######################################################################
+# Check for a command in the PATH.
+#
+sub checkcmd {
+    my ($cmd)=@_;
+    my @paths=("/usr/sbin", "/usr/local/sbin", "/sbin", "/usr/bin",
+               "/usr/local/bin", split(":", $ENV{'PATH'}));
+    for(@paths) {
+        if( -x "$_/$cmd") {
+            return "$_/$cmd";
+        }
+    }
+}
+
 #######################################################################
 # Return the pid of the server as found in the given pid file
 #
@@ -98,6 +151,112 @@ sub serverpid {
     return $PID;
 }
 
+#######################################################################
+# Memory allocation test and failure torture testing.
+#
+sub torture {
+    my $testcmd = shift;
+    my $gdbline = shift;
+
+    # remove memdump first to be sure we get a new nice and clean one
+    unlink($memdump);
+
+    # First get URL from test server, ignore the output/result
+    system($testcmd);
+
+    print " CMD: $testcmd\n" if($verbose);
+
+    # memanalyze -v is our friend, get the number of allocations made
+    my $count=0;
+    my @out = `$memanalyze -v $memdump`;
+    for(@out) {
+        if(/^Allocations: (\d+)/) {
+            $count = $1;
+            last;
+        }
+    }
+    if(!$count) {
+        print " found no allocs to make fail\n";
+        return 0;
+    }
+
+    print " $count allocations to make fail\n";
+
+    for ( 1 .. $count ) {
+        my $limit = $_;
+        my $fail;
+        my $dumped_core;
+
+        if($tortalloc && ($tortalloc != $limit)) {
+            next;
+        }
+
+        print "Fail alloc no: $limit\r" if($verbose);
+
+        # make the memory allocation function number $limit return failure
+        $ENV{'CURL_MEMLIMIT'} = $limit;
+
+        # remove memdump first to be sure we get a new nice and clean one
+        unlink($memdump);
+
+        print "**> Alloc number $limit is now set to fail <**\n" if($gdbthis);
+
+        my $ret;
+        if($gdbthis) {
+            system($gdbline)
+        }
+        else {
+            $ret = system($testcmd);
+        }
+
+        # Now clear the variable again
+        $ENV{'CURL_MEMLIMIT'} = undef;
+
+        if(-r "core") {
+            # there's core file present now!
+            print " core dumped!\n";
+            $dumped_core = 1;
+            $fail = 2;
+        }
+
+        # verify that it returns a proper error code, doesn't leak memory
+        # and doesn't core dump
+        if($ret & 255) {
+            print " system() returned $ret\n";
+            $fail=1;
+        }
+        else {
+            my @memdata=`$memanalyze $memdump`;
+            my $leak=0;
+            for(@memdata) {
+                if($_ ne "") {
+                    # well it could be other memory problems as well, but
+                    # we call it leak for short here
+                    $leak=1;
+                }
+            }
+            if($leak) {
+                print "** MEMORY FAILURE\n";
+                print @memdata;
+                print `$memanalyze -l $memdump`;
+                $fail = 1;
+            }
+        }
+        if($fail) {
+            print " Failed on alloc number $limit in test.\n",
+            " invoke with -t$limit to repeat this single case.\n";
+            stopservers();
+            exit 1;
+        }
+    }
+
+    print "torture OK\n";
+    return 0;
+
+    #stopservers();
+    #exit; # for now, we stop after these tests
+}
+
 #######################################################################
 # stop the given test server
 #
@@ -155,7 +314,7 @@ sub runhttpserver {
     $pid = checkserver ($HTTPPIDFILE);
 
     # verify if our/any server is running on this port
-    my $cmd = "$CURL -o log/verifiedserver --silent -i $HOSTIP:$HOSTPORT/verifiedserver 2>/dev/null";
+    my $cmd = "$CURL -o log/verifiedserver http://$HOSTIP:$HOSTPORT/verifiedserver 2>/dev/null";
     print "CMD; $cmd\n" if ($verbose);
     my $res = system($cmd);
 
@@ -172,7 +331,7 @@ sub runhttpserver {
     if ( $data =~ /WE ROOLZ: (\d+)/ ) {
         $pid = 0+$1;
     }
-    elsif($data) {
+    elsif($data || ($res != 7)) {
         print "RUN: Unknown HTTP server is running on port $HOSTPORT\n";
         return -2;
     }
@@ -188,6 +347,10 @@ sub runhttpserver {
     }
 
     my $flag=$debugprotocol?"-v ":"";
+    my $dir=$ENV{'srcdir'};
+    if($dir) {
+        $flag .= "-d \"$dir\" ";
+    }
     $cmd="$perl $srcdir/httpserver.pl $flag $HOSTPORT &";
     system($cmd);
     if($verbose) {
@@ -195,9 +358,9 @@ sub runhttpserver {
     }
 
     my $verified;
-    for(1 .. 5) {
+    for(1 .. 10) {
         # verify that our server is up and running:
-        my $data=`$CURL --silent -i $HOSTIP:$HOSTPORT/verifiedserver 2>/dev/null`;
+        my $data=`$CURL --silent $HOSTIP:$HOSTPORT/verifiedserver 2>/dev/null`;
 
         if ( $data =~ /WE ROOLZ: (\d+)/ ) {
             $pid = 0+$1;
@@ -205,7 +368,10 @@ sub runhttpserver {
             last;
         }
         else {
-            sleep(1);
+            if($verbose) {
+                print STDERR "RUN: Retrying HTTP server existence in 3 sec\n";
+            }
+            sleep(3);
             next;
         }
     }
@@ -228,6 +394,11 @@ sub runhttpsserver {
     my $verbose = $_[0];
     my $STATUS;
     my $RUNNING;
+
+    if(!$stunnel) {
+        return 0;
+    }
+
     my $pid=checkserver($HTTPSPIDFILE );
 
     if($pid > 0) {
@@ -239,13 +410,26 @@ sub runhttpsserver {
     }
 
     my $flag=$debugprotocol?"-v ":"";
-    my $cmd="$perl $srcdir/httpsserver.pl $flag -d $srcdir -r $HOSTPORT $HTTPSPORT &";
+    my $cmd="$perl $srcdir/httpsserver.pl $flag -s \"$stunnel\" -d $srcdir -r $HOSTPORT $HTTPSPORT &";
     system($cmd);
     if($verbose) {
         print "CMD: $cmd\n";
     }
     sleep(1);
-    $pid=checkserver($HTTPSPIDFILE);
+
+    for(1 .. 10) {
+        $pid=checkserver($HTTPSPIDFILE);
+
+        if($pid <= 0) {
+            if($verbose) {
+                print STDERR "RUN: waiting 3 sec for HTTPS server\n";
+            }
+            sleep(3);
+        }
+        else {
+            last;
+        }
+    }
 
     return $pid;
 }
@@ -264,25 +448,25 @@ sub runftpserver {
         print "RUN: Check port $FTPPORT for our own FTP server\n"
             if ($verbose);
 
-
         my $time=time();
         # check if this is our server running on this port:
-        my $data=`$CURL -m4 --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
+        my @data=`$CURL -m4 --silent ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
+        my $line;
 
         # if this took more than 2 secs, we assume it "hung" on a weird server
         my $took = time()-$time;
-        
-        if ( $data =~ /WE ROOLZ: (\d+)/ ) {
-            # this is our test server with a known pid!
-            $pid = 0+$1;
-        }
-        else {
-            if($data || ($took > 2)) {
-                # this is not a known server
-                print "RUN: Unknown server on our favourite port: $FTPPORT\n";
-                return -1;
+
+        foreach $line (@data) {
+            if ( $line =~ /WE ROOLZ: (\d+)/ ) {
+                # this is our test server with a known pid!
+                $pid = 0+$1;
             }
         }
+        if(!$pid && $data[0]) {
+            # this is not a known server
+            print "RUN: Unknown server on our favourite FTP port: $FTPPORT\n";
+            return -1;
+        }
     }
 
     if($pid > 0) {
@@ -295,9 +479,10 @@ sub runftpserver {
         }
         sleep(1);
     }
-    
+
     # now (re-)start our server:
     my $flag=$debugprotocol?"-v ":"";
+    $flag .= "-s \"$srcdir\"";
     my $cmd="$perl $srcdir/ftpserver.pl $flag $FTPPORT &";
     if($verbose) {
         print "CMD: $cmd\n";
@@ -305,21 +490,29 @@ sub runftpserver {
     system($cmd);
 
     my $verified;
-    for(1 .. 5) {
+    $pid = 0;
+    for(1 .. 10) {
         # verify that our server is up and running:
-        my $data=`$CURL --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
-
-        if ( $data =~ /WE ROOLZ: (\d+)/ ) {
-            $pid = 0+$1;
-            $verified = 1;
-            last;
+        my $line;
+        my $cmd="$CURL --silent ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null";
+        print "$cmd\n" if($verbose);
+        my @data = `$cmd`;
+        foreach $line (@data) {
+            if ( $line =~ /WE ROOLZ: (\d+)/ ) {
+                $pid = 0+$1;
+                $verified = 1;
+                last;
+            }
+        }
+        if(!$pid) {
+            if($verbose) {
+                print STDERR "RUN: Retrying FTP server existence in 3 sec\n";
+            }
+            sleep(3);
+            next;
         }
         else {
-            if($verbose) {
-                print STDERR "RUN: Retrying FTP server existance in 1 sec\n";
-            }
-            sleep(1);
-            next;
+            last;
         }
     }
     if(!$verified) {
@@ -341,6 +534,10 @@ sub runftpsserver {
     my $verbose = $_[0];
     my $STATUS;
     my $RUNNING;
+
+    if(!$stunnel) {
+        return 0;
+    }
     my $pid=checkserver($FTPSPIDFILE );
 
     if($pid > 0) {
@@ -352,14 +549,27 @@ sub runftpsserver {
     }
 
     my $flag=$debugprotocol?"-v ":"";
-    my $cmd="$perl $srcdir/ftpsserver.pl $flag -d $srcdir -r $FTPPORT $FTPSPORT &";
+    my $cmd="$perl $srcdir/ftpsserver.pl $flag -s \"$stunnel\" -d $srcdir -r $FTPPORT $FTPSPORT &";
     system($cmd);
     if($verbose) {
         print "CMD: $cmd\n";
     }
     sleep(1);
 
-    $pid=checkserver($FTPSPIDFILE );
+    for(1 .. 10) {
+
+        $pid=checkserver($FTPSPIDFILE );
+
+        if($pid <= 0) {
+            if($verbose) {
+                print STDERR "RUN: waiting 3 sec for FTPS server\n";
+            }
+            sleep(3);
+        }
+        else {
+            last;
+        }
+    }
 
     return $pid;
 }
@@ -407,7 +617,7 @@ sub filteroff {
         print OUT $_;
     }
     close(IN);
-    close(OUT);    
+    close(OUT);
     return 0;
 }
 
@@ -443,18 +653,57 @@ sub checkcurl {
 
     my $curl;
     my $libcurl;
-    my @version=`$CURL -V`;
+    my @version=`$CURL --version 2>/dev/null`;
     for(@version) {
         chomp;
 
         if($_ =~ /^curl/) {
             $curl = $_;
-
             $curl =~ s/^(.*)(libcurl.*)/$1/g;
             $libcurl = $2;
+            if($curl =~ /mingw32/) {
+                # This is a windows minw32 build, we need to translate the
+                # given path to the "actual" windows path.
 
-           if ($curl =~ /win32/)
-           {
+                my @m = `mount`;
+                my $matchlen;
+                my $bestmatch;
+                my $mount;
+
+# example mount output:
+# C:\DOCUME~1\Temp on /tmp type user (binmode,noumount)
+# c:\ActiveState\perl on /perl type user (binmode)
+# C:\msys\1.0\bin on /usr/bin type user (binmode,cygexec,noumount)
+# C:\msys\1.0\bin on /bin type user (binmode,cygexec,noumount)
+
+                foreach $mount (@m) {
+                    if( $mount =~ /(.*) on ([^ ]*) type /) {
+                        my ($mingw, $real)=($2, $1);
+                        if($pwd =~ /^$mingw/) {
+                            # the path we got from pwd starts with the path
+                            # we found on this line in the mount output
+
+                            my $len = length($real);
+                            if($len > $matchlen) {
+                                # we remember the match that is the longest
+                                $matchlen = $len;
+                                $bestmatch = $real;
+                            }
+                        }
+                    }
+                }
+                if(!$matchlen) {
+                    print "Serious error, can't find our \"real\" path!\n";
+                }
+                else {
+                    # now prepend the prefix from the mount command to build
+                    # our "actual path"
+                    $pwd = "$bestmatch$pwd";
+                }
+                $pwd =~ s#\\#/#g;
+            }
+            elsif ($curl =~ /win32/)
+            {
                # Native Windows builds don't understand the
                # output of cygwin's pwd.  It will be
                # something like /cygdrive/c/<some path>.
@@ -485,8 +734,19 @@ sub checkcurl {
                 # ssl enabled
                 $ssl_version=1;
             }
+            if($feat =~ /Largefile/i) {
+                # large file support
+                $large_file=1;
+            }
+            if($feat =~ /IDN/i) {
+                # IDN support
+                $has_idn=1;
+            }
         }
     }
+    if(!$curl) {
+        die "couldn't run '$CURL'"
+    }
 
     my $hostname=`hostname`;
     my $hosttype=`uname -a`;
@@ -497,20 +757,22 @@ sub checkcurl {
     "* Host: $hostname",
     "* System: $hosttype";
 
-    printf("* Server SSL:       %s\n", $checkstunnel?"ON":"OFF");
+    printf("* Server SSL:       %s\n", $stunnel?"ON":"OFF");
     printf("* libcurl SSL:      %s\n", $ssl_version?"ON":"OFF");
     printf("* libcurl debug:    %s\n", $curl_debug?"ON":"OFF");
+    printf("* valgrind:         %s\n", $valgrind?"ON":"OFF");
     print "***************************************** \n";
 }
 
 #######################################################################
-# substitute the variable stuff into either a joined up file or 
+# substitute the variable stuff into either a joined up file or
 # a command, in either case passed by reference
 #
 sub subVariables {
   my ($thing) = @_;
   $$thing =~ s/%HOSTIP/$HOSTIP/g;
   $$thing =~ s/%HOSTPORT/$HOSTPORT/g;
+  $$thing =~ s/%HTTPPORT/$HOSTPORT/g;
   $$thing =~ s/%HTTPSPORT/$HTTPSPORT/g;
   $$thing =~ s/%FTPPORT/$FTPPORT/g;
   $$thing =~ s/%FTPSPORT/$FTPSPORT/g;
@@ -542,7 +804,7 @@ sub singletest {
     }
 
     printf("test %03d...", $testnum);
-    
+
     for(@what) {
         my $f = $_;
         $f =~ s/\s//g;
@@ -557,8 +819,18 @@ sub singletest {
                 next;
             }
         }
+        elsif($f eq "large_file") {
+            if($large_file) {
+                next;
+            }
+        }
+        elsif($f eq "idn") {
+            if($has_idn) {
+                next;
+            }
+        }
 
-        $why = "lacks $f";
+        $why = "curl lacks $f support";
         $serverproblem = 15; # set it here
         last;
     }
@@ -583,7 +855,10 @@ sub singletest {
             # set above, a lacking prereq
         }
         elsif($serverproblem == 1) {
-            $why = "no SSL-capable server";
+            $why = "no HTTPS server";
+        }
+        elsif($serverproblem == 3) {
+            $why = "no FTPS server";
         }
         else {
             $why = "unfulfilled requirements";
@@ -591,7 +866,7 @@ sub singletest {
         $skipped++;
         $skipped{$why}++;
         $teststat[$testnum]=$why; # store reason for this test case
-        
+
         print "SKIPPED\n";
         if(!$short) {
             print "* Test $testnum: $why\n";
@@ -613,7 +888,7 @@ sub singletest {
             # of the datacheck
             chomp($replycheck[$#replycheck]);
         }
-    
+
         @reply=@replycheck;
     }
 
@@ -712,7 +987,7 @@ sub singletest {
         subVariables \$fileContent;
 #        print "DEBUG: writing file " . $filename . "\n";
         open OUTFILE, ">$filename";
-        binmode OUTFILE; # for crapage systems, use binary       
+        binmode OUTFILE; # for crapage systems, use binary
         print OUTFILE $fileContent;
         close OUTFILE;
     }
@@ -756,27 +1031,68 @@ sub singletest {
         $DBGCURL=$CMDLINE;
     }
 
+    if($valgrind) {
+        $CMDLINE = "valgrind --leak-check=yes --logfile=log/valgrind$testnum -q $CMDLINE";
+    }
+
     $CMDLINE .= "$cmdargs >>$STDOUT 2>>$STDERR";
 
     if($verbose) {
-        print "$CMDLINE\n"; 
-   }
+        print "$CMDLINE\n";
+    }
 
     print CMDLOG "$CMDLINE\n";
 
-    my $res;
-    # run the command line we built
+    unlink("core");
+
+    my $dumped_core;
+    my $cmdres;
+
+
     if($gdbthis) {
         open(GDBCMD, ">log/gdbcmd");
         print GDBCMD "set args $cmdargs\n";
         print GDBCMD "show args\n";
         close(GDBCMD);
-        system("gdb $DBGCURL -x log/gdbcmd");
-        $res =0; # makes it always continue after a debugged run
+    }
+    # run the command line we built
+    if ($torture) {
+        return torture($CMDLINE,
+                       "gdb --directory libtest $DBGCURL -x log/gdbcmd");
+    }
+    elsif($gdbthis) {
+        system("gdb --directory libtest $DBGCURL -x log/gdbcmd");
+        $cmdres=0; # makes it always continue after a debugged run
     }
     else {
-        $res = system("$CMDLINE");
-        $res /= 256;
+        $cmdres = system("$CMDLINE");
+        my $signal_num  = $cmdres & 127;
+        $dumped_core = $cmdres & 128;
+
+        if(!$anyway && ($signal_num || $dumped_core)) {
+            $cmdres = 1000;
+        }
+        else {
+            $cmdres /= 256;
+        }
+    }
+    if(!$dumped_core) {
+        if(-r "core") {
+            # there's core file present now!
+            $dumped_core = 1;
+        }
+    }
+
+    if($dumped_core) {
+        print "core dumped!\n";
+        if(0 && $gdb) {
+            print "running gdb for post-mortem analysis:\n";
+            open(GDBCMD, ">log/gdbcmd2");
+            print GDBCMD "bt\n";
+            close(GDBCMD);
+            system("gdb --directory libtest -x log/gdbcmd2 -batch $DBGCURL core ");
+     #       unlink("log/gdbcmd2");
+        }
     }
 
     # remove the special FTP command file after each test!
@@ -790,25 +1106,7 @@ sub singletest {
     my @err = getpart("verify", "errorcode");
     my $errorcode = $err[0];
 
-    if($errorcode || $res) {
-        if($errorcode == $res) {
-            $errorcode =~ s/\n//;
-            if($verbose) {
-                print " received errorcode $errorcode OK";
-            }
-            elsif(!$short) {
-                print " error OK";
-            }
-        }
-        else {
-            if(!$short) {
-                print "curl returned $res, ".(0+$errorcode)." was expected\n";
-            }
-            print " error FAILED\n";
-            return 1;
-        }
-    }
-
+    my $res;
     if (@validstdout) {
         # verify redirected stdout
         my @actual = loadarray($STDOUT);
@@ -866,7 +1164,8 @@ sub singletest {
         }
 
         for(@strip) {
-            # strip all patterns from both arrays
+            # strip off all lines that match the patterns from both arrays
+            chomp $_;
             @out = striparray( $_, \@out);
             @protstrip= striparray( $_, \@protstrip);
         }
@@ -898,16 +1197,29 @@ sub singletest {
         }
         if(!$short) {
             print " output OK";
-        }        
+        }
     }
 
-    if(!$keepoutfiles) {
-        # remove the stdout and stderr files
-        unlink($STDOUT);
-        unlink($STDERR);
-        unlink($CURLOUT); # remove the downloaded results
+    if($errorcode == $cmdres) {
+        $errorcode =~ s/\n//;
+        if($verbose) {
+            print " received exitcode $errorcode OK";
+        }
+        elsif(!$short) {
+            print " exit OK";
+        }
+    }
+    else {
+        if(!$short) {
+            print "curl returned $cmdres, ".(0+$errorcode)." was expected\n";
+        }
+        print " exit FAILED\n";
+        return 1;
+    }
 
-        unlink("$LOGDIR/upload.$testnum");  # remove upload leftovers
+    # the test succeeded, remove all log files
+    if(!$keepoutfiles) {
+        cleardir($LOGDIR);
     }
 
     unlink($FTPDCMD); # remove the instructions for this test
@@ -950,6 +1262,37 @@ sub singletest {
                 }
             }
         }
+    }
+    if($valgrind) {
+        opendir(DIR, "log") ||
+            return 0; # can't open log dir
+        my @files = readdir(DIR);
+        closedir DIR;
+        my $f;
+        my $l;
+        foreach $f (@files) {
+            if($f =~ /^valgrind$testnum/) {
+                $l = $f;
+                last;
+            }
+        }
+        my $leak;
+        open(VAL, "<$l");
+        while(<VAL>) {
+            if($_ =~ /definitely lost: (\d*) bytes/) {
+                $leak = $1;
+                last;
+            }
+        }
+        close(VAL);
+        if($leak) {
+            print " valgrind ERROR ";
+        }
+        elsif(!$short) {
+            print " valgrind OK";
+        }
+
+
     }
     if($short) {
         print "OK";
@@ -959,6 +1302,106 @@ sub singletest {
     return 0;
 }
 
+#######################################################################
+# Stop all running test servers
+sub stopservers {
+    print "Shutting down test suite servers:\n" if ($verbose);
+    for(keys %run) {
+        printf ("* kill pid for %-5s => %-5d\n", $_, $run{$_}) if($verbose);
+        stopserver($run{$_}); # the pid file is in the hash table
+    }
+}
+
+#######################################################################
+# startservers() starts all the named servers
+#
+sub startservers {
+    my @what = @_;
+    my $pid;
+    for(@what) {
+        my $what = lc($_);
+        $what =~ s/[^a-z]//g;
+        if($what eq "ftp") {
+            if(!$run{'ftp'}) {
+                $pid = runftpserver($verbose);
+                if($pid <= 0) {
+                    return 2; # error starting it
+                }
+                printf ("* pid ftp => %-5d\n", $pid) if($verbose);
+                $run{'ftp'}=$pid;
+            }
+        }
+        elsif($what eq "http") {
+            if(!$run{'http'}) {
+                $pid = runhttpserver($verbose);
+                if($pid <= 0) {
+                    return 2; # error starting
+                }
+                printf ("* pid http => %-5d\n", $pid) if($verbose);
+                $run{'http'}=$pid;
+            }
+        }
+        elsif($what eq "ftps") {
+            if(!$stunnel || !$ssl_version) {
+                # we can't run ftps tests without stunnel
+                # or if libcurl is SSL-less
+                return 3;
+            }
+            if(!$run{'ftp'}) {
+                $pid = runftpserver($verbose);
+                if($pid <= 0) {
+                    return 2; # error starting it
+                }
+                printf ("* pid ftp => %-5d\n", $pid) if($verbose);
+                $run{'ftp'}=$pid;
+            }
+            if(!$run{'ftps'}) {
+                return 2;
+
+                $pid = runftpsserver($verbose);
+                if($pid <= 0) {
+                    return 2;
+                }
+                printf ("* pid ftps => %-5d\n", $pid) if($verbose);
+                $run{'ftps'}=$pid;
+            }
+        }
+        elsif($what eq "file") {
+            # we support it but have no server!
+        }
+        elsif($what eq "https") {
+            if(!$stunnel || !$ssl_version) {
+                # we can't run https tests without stunnel
+                # or if libcurl is SSL-less
+                return 1;
+            }
+            if(!$run{'http'}) {
+                $pid = runhttpserver($verbose);
+                if($pid <= 0) {
+                    return 2; # problems starting server
+                }
+                printf ("* pid http => %-5d\n", $pid) if($verbose);
+                $run{'http'}=$pid;
+            }
+            if(!$run{'https'}) {
+                $pid = runhttpsserver($verbose);
+                if($pid <= 0) {
+                    return 2;
+                }
+                printf ("* pid https => %-5d\n", $pid) if($verbose);
+                $run{'https'}=$pid;
+            }
+        }
+        elsif($what eq "none") {
+            print "* starts no server\n" if ($verbose);
+        }
+        else {
+            warn "we don't support a server for $what";
+        }
+    }
+    return 0;
+}
+
 ##############################################################################
 # This function makes sure the right set of server is running for the
 # specified test case. This is a useful design when we run single tests as not
@@ -967,12 +1410,12 @@ sub singletest {
 # Returns:
 # 100 if this is not a test case
 # 99  if this test case has no servers specified
+# 3   if this test is skipped due to no FTPS server
 # 2   if one of the required servers couldn't be started
-# 1   if this test is skipped due to unfulfilled SSL/stunnel-requirements
+# 1   if this test is skipped due to no HTTPS server
 
 sub serverfortest {
     my ($testnum)=@_;
-    my $pid;
 
     # load the test case file definition
     if(loadtest("${TESTDIR}/test${testnum}")) {
@@ -990,78 +1433,7 @@ sub serverfortest {
         return 99;
     }
 
-    for(@what) {
-        my $what = lc($_);
-        $what =~ s/[^a-z]//g;
-        if($what eq "ftp") {
-            if(!$run{'ftp'}) {
-                $pid = runftpserver($verbose);
-                if($pid <= 0) {
-                    return 2; # error starting it
-                }
-                $run{'ftp'}=$pid;
-            }
-        }
-        elsif($what eq "http") {
-            if(!$run{'http'}) {
-                $pid = runhttpserver($verbose);
-                if($pid <= 0) {
-                    return 2; # error starting
-                }
-                $run{'http'}=$pid;
-            }
-        }
-        elsif($what eq "ftps") {
-            if(!$checkstunnel || !$ssl_version) {
-                # we can't run https tests without stunnel
-                # or if libcurl is SSL-less
-                return 1;
-            }
-            if(!$run{'ftp'}) {
-                $pid = runftpserver($verbose);
-                if($pid <= 0) {
-                    return 2; # error starting it
-                }
-                $run{'ftp'}=$pid;
-            }
-            if(!$run{'ftps'}) {
-                $pid = runftpsserver($verbose);
-                if($pid <= 0) {
-                    return 2;
-                }
-                $run{'ftps'}=$pid;
-            }
-        }
-        elsif($what eq "file") {
-            # we support it but have no server!
-        }
-        elsif($what eq "https") {
-            if(!$checkstunnel || !$ssl_version) {
-                # we can't run https tests without stunnel
-                # or if libcurl is SSL-less
-                return 1;
-            }
-            if(!$run{'http'}) {
-                $pid = runhttpserver($verbose);
-                if($pid <= 0) {
-                    return 2; # problems starting server
-                }
-                $run{'http'}=$pid;
-            }
-            if(!$run{'https'}) {
-                $pid = runhttpsserver($verbose);
-                if($pid <= 0) {
-                    return 2;
-                }
-                $run{'https'}=$pid;
-            }
-        }
-        elsif($what eq "none") {
-        }
-        else {
-            warn "we don't support a server for $what";
-        }
-    }
+    return &startservers(@what);
 }
 
 #######################################################################
@@ -1077,12 +1449,12 @@ do {
         $verbose=1;
     }
     elsif ($ARGV[0] eq "-c") {
-        # use this path to curl instead of default        
+        # use this path to curl instead of default
         $CURL=$ARGV[1];
         shift @ARGV;
     }
     elsif ($ARGV[0] eq "-d") {
-        # have the servers display protocol output 
+        # have the servers display protocol output
         $debugprotocol=1;
     }
     elsif ($ARGV[0] eq "-g") {
@@ -1093,10 +1465,28 @@ do {
         # short output
         $short=1;
     }
+    elsif($ARGV[0] eq "-n") {
+        # no valgrind
+        undef $valgrind;
+    }
+    elsif($ARGV[0] =~ /^-t(.*)/) {
+        # torture
+        $torture=1;
+        my $xtra = $1;
+
+        if($xtra =~ s/(\d+)$//) {
+            $tortalloc = $1;
+        }
+        # we undef valgrind to make this fly in comparison
+        undef $valgrind;
+    }
     elsif($ARGV[0] eq "-a") {
         # continue anyway, even if a test fail
         $anyway=1;
     }
+    elsif($ARGV[0] eq "-p") {
+        $postmortem=1;
+    }
     elsif($ARGV[0] eq "-l") {
         # lists the test case names only
         $listonly=1;
@@ -1115,7 +1505,10 @@ Usage: runtests.pl [options]
   -h       this help text
   -k       keep stdout and stderr files present after tests
   -l       list all test case names/descriptions
+  -n       No valgrind
+  -p       Print log file contents when a test fails
   -s       short output
+  -t       torture
   -v       verbose output
   [num]    like "5 6 9" or " 5 to 22 " to run those tests only
 EOHELP
@@ -1143,6 +1536,18 @@ if($testthis[0] ne "") {
     $TESTCASES=join(" ", @testthis);
 }
 
+if($valgrind) {
+    # we have found valgrind on the host, use it
+
+    # verify that we can invoke it fine
+    my $code = system("valgrind >/dev/null 2>&1");
+
+    if(($code>>8) != 1) {
+        #print "Valgrind failure, disable it\n";
+        undef $valgrind;
+    }
+}
+
 #######################################################################
 # Output curl version and host info being tested
 #
@@ -1169,7 +1574,7 @@ if ( $TESTCASES eq "all") {
 
     $TESTCASES=""; # start with no test cases
 
-    # cut off everything but the digits 
+    # cut off everything but the digits
     for(@cmds) {
         $_ =~ s/[a-z\/\.]*//g;
     }
@@ -1185,6 +1590,36 @@ if ( $TESTCASES eq "all") {
 open(CMDLOG, ">$CURLLOG") ||
     print "can't log command lines to $CURLLOG\n";
 
+#######################################################################
+
+sub displaylogcontent {
+    my ($file)=@_;
+    open(SINGLE, "<$file");
+    while(<SINGLE>) {
+        print " $_";
+    }
+    close(SINGLE);
+}
+
+sub displaylogs {
+    my ($testnum)=@_;
+    opendir(DIR, "$LOGDIR") ||
+        die "can't open dir: $!";
+    my @logs = readdir(DIR);
+    closedir DIR;
+    my $log;
+
+    print "== Contents of files in the log/ dir after test $testnum\n";
+    foreach $log (sort @logs) {
+        # the log file is not "." or ".." and contains more than zero bytes
+        if(($log !~ /\.(\.|)$/) && -s "$LOGDIR/$log") {
+            print "== Start of file $log\n";
+            displaylogcontent("$LOGDIR/$log");
+            print "== End of file $log\n";
+        }
+    }
+}
+
 #######################################################################
 # The main test-loop
 #
@@ -1209,6 +1644,10 @@ foreach $testnum (split(" ", $TESTCASES)) {
 
     if($error>0) {
         $failed.= "$testnum ";
+        if($postmortem) {
+            # display all files in log/ in a nice way
+            displaylogs($testnum);
+        }
         if(!$anyway) {
             # a test failed, abort
             print "\n - abort tests\n";
@@ -1227,15 +1666,9 @@ foreach $testnum (split(" ", $TESTCASES)) {
 #
 close(CMDLOG);
 
-#######################################################################
-# Tests done, stop the servers
-#
 
-print "Shutting down test suite servers:\n" if (!$short);
-for(keys %run) {
-    printf ("* kill pid for %-5s => %-5d\n", $_, $run{$_}) if(!$short);
-    stopserver($run{$_}); # the pid file is in the hash table
-}
+# Tests done, stop the servers
+stopservers();
 
 my $all = $total + $skipped;
 
